typedef struct //Contains generic x/y information
{
    int x, y, orientation;
}ENEMY_COORD;

typedef struct //Enemy stats, like hp/damage etc.
{
    int health;
    int damage;
    int speed_modifier;
    int vulnerabilities;
    bool hardtype;
}ENEMY_STATS;

typedef struct //Display and frameupdate data
{
    CUSTOM_IMAGE **images;
    bool hflip;
    bool vflip;
    int draw_cnt;
    int rotation;
    int to_show;
    void (*update)(int);
}ENEMY_DISPLAY;

typedef struct //This is used to prevent an enemy from being hit many times by the plasma beam
{
    int total_codes;
    int *codes;
}ENEMY_IMMUNE;

typedef struct //Basic enemy class
{
    bool active;
    int type_flag;
    ENEMY_COORD coord;
    DIMENSION bound;
    ENEMY_STATS stats;
    ENEMY_DISPLAY display;
    ENEMY_IMMUNE immune;
    void (*funcptr)(int);
    unsigned char *substructure;
    int size_of_substructure;
    int team;
    int stun_timer;
    int freeze_timer;
    int palette;
    int refreeze_timer;
    HSTREAM *Sound;
}ENEMY;

typedef struct
{
    bool active;
    int x, y;
    float xspeed, yspeed;
    float xforce, yforce;
    int rotation; //0 to 360
    int lifespan; //When 0, terminate the fireball
}RIDLEY_FIREBALL;

typedef struct
{
    bool active;
    int x, y;
    DIMENSION dim;
    int health;
    int damage;
    int speed_modifier;
    int vulnerabilities;
    CUSTOM_IMAGE *images[4];
    CUSTOM_IMAGE *fireball_img;
    int to_show;
    int to_show_timer;
    int stun_timer;
    int palette;

    bool jumping;
    bool jump_up;
    int timer;
    RIDLEY_FIREBALL Fireball[5];
    int fireball_timer;
    int team;
    HSTREAM *Sound;
    ENEMY_IMMUNE immune;
}RIDLEY;

typedef struct
{
    bool active;
    float x, y;
    float xspeed, yspeed;
    float xforce, yforce;
    fixed angle;
    int timer;
    DIMENSION dim;
    int health;
}KRAID_HOOK;

typedef struct
{
    bool active;
    float x, y;
    bool facing_left;
    int destroy_timer; //When 30, flying.  When less, counting to 0.  When 0, destroyed.
    int launch_timer; //Hooks to Kraid's side.  When 0, starts flying.
    bool stuck_in_wall;
    DIMENSION dim;
    int health;
}KRAID_NEEDLE;

typedef struct
{
    bool active;
    bool fake;
    int x, y;
    DIMENSION dim;
    int health;
    int damage;
    int speed_modifier;
    int vulnerabilities;
    CUSTOM_IMAGE *images[2];
    CUSTOM_IMAGE *hook_img;
    CUSTOM_IMAGE *needle_img;
    int to_show;
    int to_show_timer;
    int team;
    int stun_timer;
    HSTREAM *Sound;
    ENEMY_IMMUNE immune;
    int palette;
    int timer;
    int top_needle_timer, mid_needle_timer, bot_needle_timer;
    int l_hook_timer, r_hook_timer;

    bool move_left;
    int dont_move_timer;

    bool facing_left;
    KRAID_HOOK Hook[2];
    KRAID_NEEDLE Needle[3];
}KRAID;

typedef struct
{
    bool active;
    int (*create_pointer)(int, int, bool, int);
    int x, y, type;
    bool hardtype;
    int cooldown;
    int cooldown_timer;

    //For spawners that only have one enemy up at a time
    bool one_enemy_only;
    int enemy_to_watch;

    //For spawners with a proximity
    bool proximity_mode;
    int proximity_range;
}ENEMY_SPAWNER;

typedef struct
{
    int draw_x, draw_y; //Add these to the main gun's x and y to draw it there
    int spawn_x, spawn_y;//Add these to the x and y to create the shot sprite.
    bool hflip, vflip;
    CUSTOM_IMAGE *img;
    bool fire_on_this_frame;
}MINIGUN_LOOKUP;

typedef struct
{
    bool active;
    int x, y;
    DIMENSION dim;
    int timer;
    MINIGUN_LOOKUP *lookup;
    int total_lookups;
    int cur_lookup;
    int direction_code;
}MINIGUN;

typedef struct
{
    bool active;
    int damage;
    int health;
    float x, y;
    DIMENSION dim;
    float xspeed, yspeed;
    float angle;
    int freeze_timer;
    int show_freeze;
    int refreeze_timer;
}RINKA;

typedef struct
{
    bool active;
    int x, y;
    DIMENSION dim;
    DIMENSION core;
    int health;
    int size; //0 = full hp, 3 = about to die.  Refers to drawing.
    int regen_timer;
}ZEBETITE;
