void Create_Clips(int xsize, int ysize, int **roomclips)
{
    for(int x = 0; x < xsize; x ++)
    {
        for(int y = 0; y < ysize; y ++)
        {
            if(roomclips[x][y] == 0) //Empty Space
            {
                for(int xi = 0; xi < 16; xi ++)
                {
                    for(int yi = 0; yi < 16; yi ++)
                    {
                        cliparray[(x*16)+xi][(y*16)+yi] = 0x00;
                    }
                }
            }
            else if(roomclips[x][y] == 1) //Square Block
            {
                for(int xi = 0; xi < 16; xi ++)
                {
                    for(int yi = 0; yi < 16; yi ++)
                    {
                        cliparray[(x*16)+xi][(y*16)+yi] = 0xFF;
                    }
                }
            }
            else if(roomclips[x][y] == 2) //Top half only
            {
                for(int xi = 0; xi < 16; xi ++)
                {
                    for(int yi = 0; yi < 8; yi ++)
                    {
                        cliparray[(x*16)+xi][(y*16)+yi] = 0xFF;
                    }
                }
            }
        }
    }
    Secure_Clips(xsize, ysize, roomclips);
}

void Secure_Clips(int xsize, int ysize, int **roomclips) //Prevents the room from being breachable.
{
    for(int x = 0; x < xsize*16; x ++)
    {
        cliparray[x][0] = 0xFF;
        cliparray[x][ysize*16-1] = 0xFF;
    }
    for(int y = 0; y < ysize*16; y ++)
    {
        cliparray[0][y] = 0xFF;
        cliparray[xsize*16-1][y] = 0xFF;
    }
}

//Note:  Can also be used to clear areas
void Clip_Area(int lft, int top, int rgt, int bot, int value)
{
    for(int x = lft; x < rgt; x ++)
    {
        for(int y = top; y < bot; y ++)
        {
            cliparray[x][y] = value;
        }
    }
    Secure_Clips(Room->xsize, Room->ysize, Room->clips);
}

//Overload using the DIMENSION structure
void Clip_Area(DIMENSION dim, int value)
{
    for(int x = dim.lft; x < dim.rgt; x ++)
    {
        for(int y = dim.top; y < dim.bot; y ++)
        {
            cliparray[x][y] = value;
        }
    }
    Secure_Clips(Room->xsize, Room->ysize, Room->clips);
}

//Add value to clip area
void Freeze_Clip_Area(DIMENSION dim, int value)
{
    for(int x = dim.lft; x < dim.rgt; x ++)
    {
        for(int y = dim.top; y < dim.bot; y ++)
        {
            if(cliparray[x][y] < STOP_FREEZE) cliparray[x][y] = cliparray[x][y] | STOP_FREEZE;
        }
    }
    Secure_Clips(Room->xsize, Room->ysize, Room->clips);
}

//Any value which is value_to_check gets zeroed
void Unclip_Area(DIMENSION dim, int value_to_check)
{
    for(int x = dim.lft; x < dim.rgt; x ++)
    {
        for(int y = dim.top; y < dim.bot; y ++)
        {
            if(cliparray[x][y] == value_to_check) cliparray[x][y] -= value_to_check;
        }
    }
    Secure_Clips(Room->xsize, Room->ysize, Room->clips);
}

bool Scan_Clips_X(int lft, int rgt, int y, unsigned char type)
{
    if(lft < 0) lft = 0;
    if(rgt > Room->xsize*16) rgt = Room->xsize*16;
    if(y < 0) y = 0;
    else if(y > Room->ysize*16) y = Room->ysize*16;

    for(int x = lft; x < rgt; x ++)
    {
        if(cliparray[x][y] & type) return true;
    }
    return false;
}

bool Scan_Clips_Y(int top, int bot, int x, unsigned char type)
{
    if(top < 0) top = 0;
    if(bot >= Room->ysize*16) bot = (Room->ysize*16)-1;
    if(x < 0) x = 0;
    else if(x >= (Room->xsize*16)-1) x = (Room->xsize*16)-1;

    for(int y = top; y < bot; y ++)
    {
        if(cliparray[x][y] & type) return true;
    }
    return false;
}

bool Scan_Clips_Rect(int lft, int top, int rgt, int bot, unsigned char type)
{
    if(top < 0) top = 0;
    if(bot > Room->ysize*16) bot = Room->ysize*16;
    if(lft < 0) lft = 0;
    if(rgt > Room->xsize*16) rgt = Room->xsize*16;

    for(int x = lft; x < rgt; x ++)
    {
        for(int y = top; y < bot; y ++)
        {
            if(cliparray[x][y] & type) return true;
        }
    }
    return false;
}

bool rect_clip(DIMENSION dim1, int lft2, int top2, int rgt2, int bot2)
{
    if(
    (((dim1.lft >= lft2) AND (dim1.lft <= rgt2))  OR
    (( dim1.rgt >= lft2) AND (dim1.rgt <= rgt2))  OR
    (( dim1.lft >= lft2) AND (dim1.rgt <= rgt2))  OR
    (( dim1.lft <= lft2) AND (dim1.rgt >= rgt2))) AND
    (((dim1.top >= top2) AND (dim1.top <= bot2))  OR
    (( dim1.bot >= top2) AND (dim1.bot <= bot2))  OR
    (( dim1.top >= top2) AND (dim1.bot <= bot2))  OR
    (( dim1.top <= top2) AND (dim1.bot >= bot2))))
    {
        return true;
    }
    return false;
}

bool rect_clip(DIMENSION dim1, DIMENSION dim2)
{
    if(
    (((dim1.lft >= dim2.lft) AND (dim1.lft <= dim2.rgt))  OR
    (( dim1.rgt >= dim2.lft) AND (dim1.rgt <= dim2.rgt))  OR
    (( dim1.lft >= dim2.lft) AND (dim1.rgt <= dim2.rgt))  OR
    (( dim1.lft <= dim2.lft) AND (dim1.rgt >= dim2.rgt))) AND
    (((dim1.top >= dim2.top) AND (dim1.top <= dim2.bot))  OR
    (( dim1.bot >= dim2.top) AND (dim1.bot <= dim2.bot))  OR
    (( dim1.top >= dim2.top) AND (dim1.bot <= dim2.bot))  OR
    (( dim1.top <= dim2.top) AND (dim1.bot >= dim2.bot))))
    {
        return true;
    }
    return false;
}
