////////////////////////////////////////////////////////////////////////////////
//  FireHorses
//
//  FireHorse Substructure
//
//  0 - State.  0 = Going down, 1 = waiting down, 2 = Going up, 3 = firing sequence.
//  1 - State timer;
////////////////////////////////////////////////////////////////////////////////
int Create_Firehorse(int x, int y, bool hardtype)
{
    for(int i = 0; i < total_enemies; i ++) //Scan the old enemies to see if one is unused
    {
        if(!Enemies[i]->active)
        {
            Place_Firehorse_In_Slot(i, x, y, hardtype);
            return i;
        }
    }
    Enemies = (ENEMY **)realloc(Enemies, sizeof(ENEMY *) * (total_enemies + 1));
    Enemies[total_enemies] = (ENEMY *)malloc(sizeof(ENEMY));
    Enemies[total_enemies]->substructure = NULL;
    Enemies[total_enemies]->display.images = NULL;
    Enemies[total_enemies]->immune.codes = NULL;

    Place_Firehorse_In_Slot(total_enemies, x, y, hardtype);
    total_enemies ++;
    return total_enemies - 1;
}

void Place_Firehorse_In_Slot(int i, int x, int y, bool hardtype)
{
    Enemies[i]->active = true;
    Enemies[i]->type_flag = GEEMER;
    Enemies[i]->funcptr = &Update_Firehorse;
    if(Enemies[i]->substructure) free(Enemies[i]->substructure);
    Enemies[i]->size_of_substructure = 2;
    Enemies[i]->substructure = (unsigned char*)malloc(sizeof(unsigned char) * Enemies[i]->size_of_substructure);
    Enemies[i]->substructure[0] = 1;
    Enemies[i]->substructure[1] = 40;
    Enemies[i]->coord.x = x;
    Enemies[i]->coord.y = y;
    Enemies[i]->coord.orientation = RIGHT;
    Enemies[i]->stats.health = 8;
    if(hardtype) Enemies[i]->stats.health *= 2;
    Enemies[i]->stats.hardtype = hardtype;
    Enemies[i]->stats.damage = 5;
    Enemies[i]->stats.speed_modifier = 1;
    Enemies[i]->stats.vulnerabilities = BEAM_ALL;
    if(Enemies[i]->display.images) free(Enemies[i]->display.images);
    Enemies[i]->display.images = (CUSTOM_IMAGE **)malloc(sizeof(CUSTOM_IMAGE *) * 2);
    Enemies[i]->display.images[0] = Enemy_Images[29];
    Enemies[i]->display.images[1] = Enemy_Images[30];
    Enemies[i]->display.to_show = 0;
    Enemies[i]->display.rotation = 0;
    Enemies[i]->display.hflip = false;
    Enemies[i]->display.vflip = false;
    Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->team = 1;
    Enemies[i]->Sound = &Audio.Injured_Large;
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 14;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 32;
    Enemies[i]->stun_timer = 0;
    Enemies[i]->freeze_timer = 0;
    Enemies[i]->refreeze_timer = 0;
    Enemies[i]->palette = 0;
    Add_To_Target_List(TARGET_ENEMY, i);

    if(Enemies[i]->immune.codes) free(Enemies[i]->immune.codes);
    Enemies[i]->immune.codes = NULL;
    Enemies[i]->immune.total_codes = 0;
}

void Update_Firehorse(int i)
{
    if(Enemies[i]->stats.health <= 0) //Firehorse got killed, bound, and roasted on a spit :(
    {
        Create_Random_Item(Enemies[i]->bound, LARGETYPE, false);
        int spritenum = Create_Sprite(((Enemies[i]->bound.lft+Enemies[i]->bound.rgt)/2)-16, ((Enemies[i]->bound.top+Enemies[i]->bound.bot)/2)-16, 3, 2, 6);
        if(spritenum != -1) Imagize_Sprite_Explosion(spritenum);
        Enemies[i]->active = false;
        Remove_From_Target_List(&Enemies[i]->stats.health);
        Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        return;
    }
    if(Enemies[i]->stun_timer > 0)
    {
        Enemies[i]->stun_timer --;
        return;
    }
    if(Enemies[i]->freeze_timer > 0)
    {
        Freeze_Clip_Area(Enemies[i]->bound, STOP_FREEZE);
        Enemies[i]->freeze_timer --;
        if(Enemies[i]->freeze_timer == 0) Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        if(Enemies[i]->freeze_timer < 50)
        {
            if((Enemies[i]->freeze_timer % 4) < 2)
            {
                Enemies[i]->palette = 2;
            }
            else
            {
                if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
                else Enemies[i]->palette = 0;
            }
        }
        else
        {
            Enemies[i]->palette = 2;
        }
        return;
    }
    if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
    else Enemies[i]->palette = 0;

    Enemies[i]->display.to_show = 0;

    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 14;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 32;

    if(Enemies[i]->substructure[0] == 0) //Going down
    {
        Enemies[i]->substructure[1] --;
        Enemies[i]->coord.y ++;
        Enemies[i]->bound.top ++;
        Enemies[i]->bound.bot ++;
        if(Enemies[i]->substructure[1] <= 0)
        {
            Enemies[i]->substructure[0] = 1;
            Enemies[i]->substructure[1] = 80;
        }
    }
    else if(Enemies[i]->substructure[0] == 1) //Waiting to go up
    {
        Enemies[i]->substructure[1] --;
        if(Enemies[i]->substructure[1] <= 0)
        {
            Enemies[i]->substructure[1] = 16;
            Enemies[i]->substructure[0] = 2;
        }
    }
    else if(Enemies[i]->substructure[0] == 2) //Going up
    {
        Enemies[i]->substructure[1] --;
        Enemies[i]->coord.y --;
        Enemies[i]->bound.top --;
        Enemies[i]->bound.bot --;
        if(Enemies[i]->substructure[1] <= 0)
        {
            Enemies[i]->substructure[0] = 3;
            Enemies[i]->substructure[1] = 176;
        }
    }
    else if(Enemies[i]->substructure[0] == 3) //Firing
    {
        int factor = 3;
        Enemies[i]->substructure[1] --;
        if(Enemies[i]->substructure[1] <= 0)
        {
            Enemies[i]->substructure[1] = 16;
            Enemies[i]->substructure[0] = 0;
        }
        else if(Enemies[i]->substructure[1] < factor * 8) //Wait
        {
            Enemies[i]->display.to_show = 0;
        }
        else if(Enemies[i]->substructure[1] < factor * 12) //Fire
        {
            if(Enemies[i]->substructure[1] == factor * 10)
            {
                int q = Create_Sprite(Enemies[i]->bound.rgt, Enemies[i]->bound.top-13, 1);
                Imagize_Sprite_Firehorse(q, RIGHT);
            }
            Enemies[i]->display.to_show = 1;
        }
        else if(Enemies[i]->substructure[1] < factor * 20) //Wait
        {
            Enemies[i]->display.to_show = 0;
        }
        else if(Enemies[i]->substructure[1] < factor * 24) //Fire
        {
            if(Enemies[i]->substructure[1] == factor * 22)
            {
                int q = Create_Sprite(Enemies[i]->bound.rgt, Enemies[i]->bound.top-13, 1);
                Imagize_Sprite_Firehorse(q, RIGHT);
            }
            Enemies[i]->display.to_show = 1;
        }
        else if(Enemies[i]->substructure[1] < factor * 32) //Wait
        {
            Enemies[i]->display.to_show = 0;
        }
        else if(Enemies[i]->substructure[1] < factor * 36) //Fire
        {
            if(Enemies[i]->substructure[1] == factor * 34)
            {
                int q = Create_Sprite(Enemies[i]->bound.rgt, Enemies[i]->bound.top-13, 1);
                Imagize_Sprite_Firehorse(q, RIGHT);
            }
            Enemies[i]->display.to_show = 1;
        }
        else //Wait
        {
            Enemies[i]->display.to_show = 0;
        }
    }
}
