////////////////////////////////////////////////////////////////////////////////
//  Gerutas
//
//  Geruta Substructure
//  0 - Distance to dive, low byte
//  1 - Distance to dive, high byte
//  2 - State.  0 = Going up.  1 = Diving.
//  3 - Direction.  0 = left, 1 = down, 2 = right.
//  4 - Pausing timer.  If > 0, don't move.
//  5 - Speed timer.  0-10 = 1, 11-20 = 2, above is 3.
//
////////////////////////////////////////////////////////////////////////////////

// 0 = Geruta
// 1 = Holtz
int Create_Geruta(int x, int y, bool hardtype, int type)
{
    for(int i = 0; i < total_enemies; i ++) //Scan the old enemies to see if one is unused
    {
        if(!Enemies[i]->active)
        {
            Place_Geruta_In_Slot(i, x, y, hardtype, type);
            return i;
        }
    }
    Enemies = (ENEMY **)realloc(Enemies, sizeof(ENEMY *) * (total_enemies + 1));
    Enemies[total_enemies] = (ENEMY *)malloc(sizeof(ENEMY));
    Enemies[total_enemies]->substructure = NULL;
    Enemies[total_enemies]->display.images = NULL;
    Enemies[total_enemies]->immune.codes = NULL;

    Place_Geruta_In_Slot(total_enemies, x, y, hardtype, type);
    total_enemies ++;
    return total_enemies - 1;
}

void Place_Geruta_In_Slot(int i, int x, int y, bool hardtype, int type)
{
    Enemies[i]->active = true;
    Enemies[i]->type_flag = GEEMER;
    Enemies[i]->funcptr = &Update_Geruta;
    if(Enemies[i]->substructure) free(Enemies[i]->substructure);
    Enemies[i]->size_of_substructure = 6;
    Enemies[i]->substructure = (unsigned char*)malloc(sizeof(unsigned char) * Enemies[i]->size_of_substructure);
    Enemies[i]->substructure[0] = 0;
    Enemies[i]->substructure[1] = 0;
    Enemies[i]->substructure[2] = 0;
    Enemies[i]->substructure[3] = 0;
    Enemies[i]->substructure[4] = 0;
    Enemies[i]->substructure[5] = 0;
    Enemies[i]->coord.x = x;
    Enemies[i]->coord.y = y;
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 16;
    Enemies[i]->coord.orientation = RIGHT;
    Enemies[i]->stats.health = 8;
    if(hardtype) Enemies[i]->stats.health *= 2;
    Enemies[i]->stats.hardtype = hardtype;
    Enemies[i]->stats.damage = 10;
    Enemies[i]->stats.speed_modifier = 1;
    Enemies[i]->stats.vulnerabilities = BEAM_ALL;
    if(Enemies[i]->display.images) free(Enemies[i]->display.images);
    Enemies[i]->display.images = (CUSTOM_IMAGE **)malloc(sizeof(CUSTOM_IMAGE *) * 4);
    if(type == 0) //Geruta
    {
        Enemies[i]->display.images[0] = Enemy_Images[13];
        Enemies[i]->display.images[1] = Enemy_Images[14];
        Enemies[i]->display.images[2] = Enemy_Images[15];
        Enemies[i]->display.images[3] = Enemy_Images[16];
    }
    else //Holtz
    {
        Enemies[i]->display.images[0] = Enemy_Images[17];
        Enemies[i]->display.images[1] = Enemy_Images[18];
        Enemies[i]->display.images[2] = Enemy_Images[19];
        Enemies[i]->display.images[3] = Enemy_Images[20];
    }
    Enemies[i]->display.to_show = 0;
    Enemies[i]->display.rotation = 0;
    Enemies[i]->display.hflip = false;
    Enemies[i]->display.vflip = false;
    Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->team = 1;
    Enemies[i]->Sound = &Audio.Injured_Large;
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 16;
    Enemies[i]->stun_timer = 0;
    Enemies[i]->freeze_timer = 0;
    Enemies[i]->refreeze_timer = 0;
    Add_To_Target_List(TARGET_ENEMY, i);

    if(Enemies[i]->immune.codes) free(Enemies[i]->immune.codes);
    Enemies[i]->immune.codes = NULL;
    Enemies[i]->immune.total_codes = 0;
}

void Update_Geruta(int i)
{
    if(Enemies[i]->stats.health <= 0) //Geruta got smoked :(
    {
        Create_Random_Item(Enemies[i]->bound, LARGETYPE, false);
        int spritenum = Create_Sprite(((Enemies[i]->bound.lft+Enemies[i]->bound.rgt)/2)-16, ((Enemies[i]->bound.top+Enemies[i]->bound.bot)/2)-16, 3, 2, 6);
        if(spritenum != -1) Imagize_Sprite_Explosion(spritenum);
        Enemies[i]->active = false;
        Remove_From_Target_List(&Enemies[i]->stats.health);
        Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        return;
    }
    if(Enemies[i]->stun_timer > 0)
    {
        Enemies[i]->stun_timer --;
        if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 0;
        else Enemies[i]->palette = 1;
        return;
    }
    if(Enemies[i]->freeze_timer > 0)
    {
        Freeze_Clip_Area(Enemies[i]->bound, STOP_FREEZE);
        Enemies[i]->freeze_timer --;
        if(Enemies[i]->freeze_timer == 0) Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        if(Enemies[i]->freeze_timer < 50)
        {
            if((Enemies[i]->freeze_timer % 4) < 2)
            {
                Enemies[i]->palette = 2;
            }
            else
            {
                if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
                else Enemies[i]->palette = 0;
            }
        }
        else
        {
            Enemies[i]->palette = 2;
        }
        return;
    }
    if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
    else Enemies[i]->palette = 0;

    Enemies[i]->display.draw_cnt ++;
    if(Enemies[i]->display.draw_cnt > 3) Enemies[i]->display.draw_cnt = 0;

    if(Enemies[i]->substructure[2] == 1) Enemies[i]->display.to_show = Enemies[i]->display.draw_cnt/2;
    else Enemies[i]->display.to_show = Enemies[i]->display.draw_cnt/2 + 2;

    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 16;

    if(Enemies[i]->substructure[4] > 0)
    {
        Enemies[i]->substructure[4] --;
    }
    else
    {
        Enemies[i]->substructure[5] ++;
        int mcspeed;
        if(Enemies[i]->substructure[5] < 10) mcspeed = 1;
        else if(Enemies[i]->substructure[5] < 20) mcspeed = 2;
        else mcspeed = 3;

        if(Enemies[i]->substructure[2] == 0) //Rising
        {
            Move_Enemy(i, Enemies[i]->substructure[3]-1, -mcspeed);
            if(Scan_Clips_X(Enemies[i]->bound.lft+1, Enemies[i]->bound.rgt-1, Enemies[i]->bound.top-1, 0x02)) //Touchdown
            {
                Enemies[i]->substructure[4] = 10;
                Enemies[i]->substructure[2] = 1;
                Enemies[i]->substructure[5] = 0;
            }
        }
        else if(Enemies[i]->substructure[2] == 1) //Diving
        {
            mcspeed = 3;
            if(Enemies[i]->substructure[1] == 0)
            {
                if(Enemies[i]->substructure[0] < 20) mcspeed = 2;
                if(Enemies[i]->substructure[0] < 10) mcspeed = 1;
            }
            for(int q = 0; q < mcspeed; q ++)
            {
                Enemies[i]->substructure[0] --;
                if(Enemies[i]->substructure[0] == 0 AND Enemies[i]->substructure[1] > 0)
                {
                    Enemies[i]->substructure[1] --;
                    Enemies[i]->substructure[0] = 255;
                }
                else if(Enemies[i]->substructure[0] == 0 AND Enemies[i]->substructure[1] == 0)
                {
                    break;
                }
            }
            if(Scan_Clips_X(Enemies[i]->bound.lft+1, Enemies[i]->bound.rgt-1, Enemies[i]->bound.bot+1, 0x02))
            {
                Enemies[i]->substructure[0] = 0;
                Enemies[i]->substructure[1] = 0;
            }

            if(Enemies[i]->substructure[0] == 0 AND Enemies[i]->substructure[1] == 0) //Stop
            {
                Enemies[i]->substructure[4] = 0;
                Enemies[i]->substructure[2] = 0;
                Enemies[i]->substructure[5] = 0;
            }
            else if(Scan_Clips_X(Enemies[i]->bound.lft+1, Enemies[i]->bound.rgt-1, Enemies[i]->bound.top-1, 0x02))//Still on the roof
            {
                if(samvar.dim.bot < Enemies[i]->bound.top) Enemies[i]->substructure[3] = 1;
                else if(samvar.dim.rgt < Enemies[i]->bound.lft) Enemies[i]->substructure[3] = 0;
                else if(samvar.dim.lft > Enemies[i]->bound.rgt) Enemies[i]->substructure[3] = 2;
                else Enemies[i]->substructure[3] = 1;

                Enemies[i]->substructure[0] = ((samvar.dim.top+16) - Enemies[i]->bound.bot);
                Enemies[i]->substructure[1] = ((samvar.dim.top+16) - Enemies[i]->bound.bot) >> 8;
                if(Enemies[i]->substructure[0] < 30 AND Enemies[i]->substructure[1] == 0 OR samvar.dim.bot < Enemies[i]->bound.bot)
                {
                    Enemies[i]->substructure[0] = 30;
                    Enemies[i]->substructure[1] = 0;
                }
                Move_Enemy(i, Enemies[i]->substructure[3]-1, mcspeed);
            }
            else
            {
                Move_Enemy(i, Enemies[i]->substructure[3]-1, mcspeed);
            }
        }
    }
}
