//The item types are as follows
//
// 0  - Small energy
// 1  - Large energy
// 2  - Missiles
// 3  - Super Missiles (Not used yet)
// 4  - Power Bombs (Not used yet)
//
// If the fanfare value is true, these types are used instead
//
// 0  - Missiles
// 1  - Energy Tank
// 2  - Morph Ball
// 3  - Bombs
// 4  - Spring Ball
// 5  - Screw Attack
// 6  - High Jump Boots
// 7  - Space Jump
// 8  - Speed Booster
// 9  - Varia Suit
// 10 - Gravity Suit
// 11 - Long Beam
// 12 - Charge Beam
// 13 - Ice Beam
// 14 - Wave Beam
// 15 - Spazer Beam
// 16 - Plasma Beam

void Create_Random_Item(DIMENSION dim, int type, bool super_type)
{
    if(super_type AND samvar.missiles < 85)
    {
        Create_Item(((dim.lft+dim.rgt)/2)-4, ((dim.top+dim.bot)/2)-4, 2, super_type);
        return;
    }
    int random = rnd(1, 99);
    if(random < 31)
    {
        if(random < 11) //10% chance on kill to drop missiles
        {
            if(samvar.missiles_max > 0) Create_Item(((dim.lft+dim.rgt)/2)-4, ((dim.top+dim.bot)/2)-4, 2, super_type);
            return;
        }
        else if(random < 21) //10% chance on kill to drop small energy
        {
            Create_Item(((dim.lft+dim.rgt)/2)-4, ((dim.top+dim.bot)/2)-4, 0, super_type);
            return;
        }
        else if(random < 31) //10% chance to drop large energy, becomes small if SMALLTYPE is true
        {
            if(type == SMALLTYPE) Create_Item(((dim.lft+dim.rgt)/2)-4, ((dim.top+dim.bot)/2)-4, 0, super_type);
            else Create_Item(((dim.lft+dim.rgt)/2)-4, ((dim.top+dim.bot)/2)-4, 1, super_type);
            return;
        }
    }
}

void Create_Item(int x, int y, int type, bool super_type) //Overload for the non-fanfare items
{
    for(int i = 0; i < total_items; i ++)
    {
        if(!Items[i].active)
        {
            Place_Item_In_Slot_No_Fanfare(i, x, y, type, super_type);
            return;
        }
    }
    Items = (ITEM *)realloc(Items, sizeof(ITEM) * (total_items + 1));
    Items[total_items].string = NULL;
    Items[total_items].bmp0 = NULL;
    Items[total_items].bmp1 = NULL;

    Place_Item_In_Slot_No_Fanfare(total_items, x, y, type, super_type);

    total_items ++;
}

int Create_Item(int x, int y, int type, char string[], int position) //Overload for the fanfare items
{
    for(int i = 0; i < total_items; i ++)
    {
        if(!Items[i].active)
        {
            Place_Item_In_Slot(i, x, y, type, string, position);
            return i;
        }
    }
    Items = (ITEM *)realloc(Items, sizeof(ITEM) * (total_items + 1));
    Items[total_items].string = NULL;
    Items[total_items].bmp0 = NULL;
    Items[total_items].bmp1 = NULL;

    Place_Item_In_Slot(total_items, x, y, type, string, position);

    total_items ++;
    return total_items-1;
}

void Place_Item_In_Slot_No_Fanfare(int i, int x, int y, int type, bool super_type)
{
    Items[i].active = true;
    Items[i].fanfare = false;
    Items[i].timer = 0;
    Items[i].x = x;
    Items[i].y = y;
    Items[i].type = type;
    Items[i].expiry = 600;
    Items[i].super_type = super_type;
    if(Items[i].string) free(Items[i].string);
    Items[i].string = NULL;

    if(type == 0 OR type == 1)
    {
        Items[i].dim.lft = x;
        Items[i].dim.top = y;
        Items[i].dim.rgt = x + 8;
        Items[i].dim.bot = y + 8;
    }
    else if(type == 2)
    {
        Items[i].dim.lft = x;
        Items[i].dim.top = y;
        Items[i].dim.rgt = x + 8;
        Items[i].dim.bot = y + 12;
    }
}

void Place_Item_In_Slot(int i, int x, int y, int type, char string[], int position)
{
    Items[i].active = true;
    Items[i].fanfare = true;
    Items[i].x = x;
    Items[i].y = y;
    Items[i].timer = 0;
    Items[i].type = type;
    Items[i].expiry = -1;
    Items[i].dim.lft = x;
    Items[i].dim.top = y;
    Items[i].dim.rgt = x + 16;
    Items[i].dim.bot = y + 16;
    Items[i].breakable_hook = -1;
    if(Items[i].string) free(Items[i].string);
    Items[i].string = (char *)malloc(sizeof(char) * (strlen(string) + 1));
    Init_String(Items[i].string, string, strlen(string));
    Items[i].bmp0 = Fanfare_Item[type*2];
    Items[i].bmp1 = Fanfare_Item[type*2+1];
    Items[i].position = position;
}

void Update_Items()
{
    for(int i = 0; i < total_items; i ++)
    {
        if(Items[i].active)
        {
            Items[i].timer ++;
            if(Items[i].timer >= 6) Items[i].timer = 0;
            if(Items[i].expiry > 0) Items[i].expiry --;
            if(Items[i].expiry == 0) Items[i].active = false;

            if(rect_clip(samvar.dim, Items[i].dim))
            {
                if(Items[i].fanfare == false)
                {
                    Items[i].active = false;
                    if(Items[i].type == 0)
                    {
                        samvar.energy += 5;
                        if(Items[i].super_type) samvar.energy += 45;
                        BASS_ChannelPlay(Audio.Get_Energy, true);
                    }
                    else if(Items[i].type == 1)
                    {
                        samvar.energy += 20;
                        if(Items[i].super_type) samvar.energy += 80;
                        BASS_ChannelPlay(Audio.Get_Energy, true);
                    }
                    else if(Items[i].type == 2)
                    {
                        samvar.missiles += 2;
                        if(Items[i].super_type) samvar.missiles += 18;
                        BASS_ChannelPlay(Audio.Get_Missile, true);
                    }
                }
                else if(Items[i].fanfare == true AND (Items[i].breakable_hook == -1 OR Breakables[Items[i].breakable_hook].show_item))
                {
                    Cut_All_Audio_Switches();
                    kill_audio = true;
                    BASS_ChannelPlay(Audio.Get_Item_Music, true);
                    item_lockout_timer = 300;
                    Items[i].active = false;
                    Add_Saved_Object(Saveout, Items[i].position);
                    if(Items[i].type == 0) //Missiles
                    {
                        samvar.missiles_max += 5;
                        samvar.missiles += 5;
                    }
                    else if(Items[i].type == 1) //Energy Tank
                    {
                        samvar.energy_max += 100;
                        samvar.energy = samvar.energy_max;
                    }
                    else if(Items[i].type ==  2)
                    {
                        samvar.Collected.morph_ball = true;
                        samvar.Using.morph_ball = true;
                    }
                    else if(Items[i].type ==  3)
                    {
                        samvar.Collected.bombs = true;
                        samvar.Using.bombs = true;
                    }
                    else if(Items[i].type ==  4)
                    {
                        samvar.Collected.spring_ball = true;
                        samvar.Using.spring_ball = true;
                    }
                    else if(Items[i].type ==  5)
                    {
                        samvar.Collected.screw_attack = true;
                        samvar.Using.screw_attack = true;
                    }
                    else if(Items[i].type ==  6)
                    {
                        samvar.Collected.high_jump = true;
                        samvar.Using.high_jump = true;
                    }
                    else if(Items[i].type ==  7)
                    {
                        samvar.Collected.space_jump = true;
                        samvar.Using.space_jump = true;
                    }
                    else if(Items[i].type ==  8)
                    {
                        samvar.Collected.speed_booster = true;
                        samvar.Using.speed_booster = true;
                    }
                    else if(Items[i].type ==  9)
                    {
                        samvar.Collected.varia_suit = true;
                        samvar.Using.varia_suit = true;
                        if(!samvar.Using.gravity_suit)
                        {
                            if(samvar.missile_mode) samvar.palette = 3;
                            else samvar.palette = 2;
                            samvar.qpalette = samvar.palette;
                        }
                    }
                    else if(Items[i].type == 10)
                    {
                        samvar.Collected.gravity_suit = true;
                        samvar.Using.gravity_suit = true;
                        if(samvar.missile_mode) samvar.palette = 5;
                        else samvar.palette = 4;
                        samvar.qpalette = samvar.palette;
                    }
                    else if(Items[i].type == 11)
                    {
                        samvar.Collected.long_beam = true;
                        samvar.Using.long_beam = true;
                    }
                    else if(Items[i].type == 12)
                    {
                        samvar.Collected.charge_beam = true;
                        samvar.Using.charge_beam = true;
                    }
                    else if(Items[i].type == 13)
                    {
                        samvar.Collected.ice_beam = true;
                        samvar.Using.ice_beam = true;
                    }
                    else if(Items[i].type == 14)
                    {
                        samvar.Collected.wave_beam = true;
                        samvar.Using.wave_beam = true;
                    }
                    else if(Items[i].type == 15)
                    {
                        samvar.Collected.spazer_beam = true;
                        samvar.Using.spazer_beam = true;
                    }
                    else if(Items[i].type == 16)
                    {
                        samvar.Collected.plasma_beam = true;
                        samvar.Using.plasma_beam = true;
                    }
                }
            }
        }
    }
}

void Hook_Item_To_Breakable(int i, int b)
{
    Items[i].breakable_hook = b;
}

void Update_All_Collected_Switches()
{
    samvar.Using.long_beam = samvar.Collected.long_beam;
    samvar.Using.charge_beam = samvar.Collected.charge_beam;
    samvar.Using.spazer_beam = samvar.Collected.spazer_beam;
    samvar.Using.plasma_beam = samvar.Collected.plasma_beam;
    samvar.Using.helix_beam = samvar.Collected.helix_beam;

    samvar.Using.ice_beam = samvar.Collected.ice_beam;
    samvar.Using.burn_beam = samvar.Collected.burn_beam;

    samvar.Using.wave_beam = samvar.Collected.wave_beam;
    samvar.Using.shock_beam = samvar.Collected.shock_beam;

    samvar.Using.varia_suit = samvar.Collected.varia_suit;
    samvar.Using.gravity_suit = samvar.Collected.gravity_suit;

    samvar.Using.morph_ball = samvar.Collected.morph_ball;
    samvar.Using.bombs = samvar.Collected.bombs;
    samvar.Using.spring_ball = samvar.Collected.spring_ball;
    samvar.Using.screw_attack = samvar.Collected.screw_attack;

    samvar.Using.high_jump = samvar.Collected.high_jump;
    samvar.Using.space_jump = samvar.Collected.space_jump;
    samvar.Using.speed_booster = samvar.Collected.speed_booster;

    /*if(samvar.Using.wave_beam AND samvar.Using.shock_beam) samvar.Using.shock_beam = false;
    if(samvar.Using.spazer_beam)
    {
        if(samvar.Using.ice_beam AND samvar.Using.burn_beam) samvar.Using.burn_beam = false;
        samvar.Using.plasma_beam = false;
        samvar.Using.helix_beam = false;
    }
    if(samvar.Using.plasma_beam)
    {
        samvar.Using.burn_beam = false;
        samvar.Using.shock_beam = false;
        samvar.Using.spazer_beam = false;
        samvar.Using.helix_beam = false;
    }
    if(samvar.Using.helix_beam)
    {
        samvar.Using.wave_beam = false;
        samvar.Using.plasma_beam = false;
        samvar.Using.spazer_beam = false;
    }*/
}

void Update_Saved_Switch(bool *use, bool *col, int sav)
{
    if(sav == 3)
    {
        *use = 1;
        *col = 1;
    }
    else if(sav == 1)
    {
        *use = 0;
        *col = 1;
    }
    else if(sav == 0)
    {
        *use = 0;
        *col = 0;
    }
}

void Clear_Collected_Items()
{
    for(int i = 0; i < Saveout->objects_saved; i ++)
    {
        if(!strcmp(Saveout->Object_Status_Save[i].roomname, curlev_name))
        {
            for(int o = 0; o < total_items; o ++)
            {
                if(Saveout->Object_Status_Save[i].object_pos == Items[o].position)
                {
                    Items[o].active = false;
                    printf("Cleared item %i\n", Items[o].position);
                }
            }
        }
    }
}

// 0  - Missiles
// 1  - Energy Tank
// 2  - Morph Ball
// 3  - Bombs
// 4  - Spring Ball
// 5  - Screw Attack
// 6  - High Jump Boots
// 7  - Space Jump
// 8  - Speed Booster
// 9  - Varia Suit
// 10 - Gravity Suit
// 11 - Long Beam
// 12 - Charge Beam
// 13 - Ice Beam
// 14 - Wave Beam
// 15 - Spazer Beam
// 16 - Plasma Beam
