//Kraid has 384 hp
void Init_Kraid(int x, int y, bool fake)
{
    KRAID_HOOK Hook[2];
    KRAID_NEEDLE Needle[3];
    Kraid.active = true;
    Kraid.fake = fake;
    Kraid.x = x * 16;
    Kraid.y = (y * 16);
    Kraid.dim.lft = Kraid.x;
    Kraid.dim.rgt = Kraid.x + 24;
    Kraid.dim.top = Kraid.y;
    Kraid.dim.bot = Kraid.y + 32;
    if(!fake)
    {
        Kraid.health = 384;
        Kraid.damage = 12;
    }
    else
    {
        Kraid.health = 160;
        Kraid.damage = 6;
    }
    Kraid.speed_modifier = 1;
    Kraid.vulnerabilities = BEAM_DAMAGE + BEAM_MISSILE + BEAM_BOMB + BEAM_BURN;
    Kraid.images[0] = Create_Image_From_MCI(Master_MCI, "Kraid0", 0);
    Kraid.images[1] = Create_Image_From_MCI(Master_MCI, "Kraid1", 0);
    Kraid.hook_img = Create_Image_From_MCI(Master_MCI, "Kraid_Hook", 0);
    Kraid.needle_img = Create_Image_From_MCI(Master_MCI, "Kraid_Spike", 0);
    Kraid.top_needle_timer = rnd(60, 60);
    Kraid.mid_needle_timer = rnd(60, 60);
    Kraid.bot_needle_timer = rnd(60, 60);
    Kraid.to_show = 0;
    Kraid.to_show_timer = 0;
    Kraid.facing_left = false;
    Kraid.team = 1;
    Kraid.Sound = &Audio.Injured_Boss;
    Kraid.stun_timer = 0;
    Kraid.immune.codes = NULL;
    Kraid.immune.total_codes = 0;
    Add_To_Target_List(TARGET_KRAID, 0);
}

void Destroy_Kraid()
{
    Kraid.active = false;
    if(Kraid.immune.codes) free(Kraid.immune.codes);
    Kraid.immune.codes = NULL;
    Remove_From_Target_List(&Kraid.health);
    if(!Kraid.fake) samvar.kraid_dead = true;
}

void Immunize_Kraid_From_Beam(int beamcode)
{
    for(int e = 0; e < Kraid.immune.total_codes; e ++)
    {
        if(Kraid.immune.codes[e] == beamcode) //Already immune to this one
        {
            return;
        }
    }
    //If the beam isn't already in the array, add it
    Kraid.immune.codes = (int *)realloc(Kraid.immune.codes, sizeof(int) * (Kraid.immune.total_codes+1));
    Kraid.immune.codes[Kraid.immune.total_codes] = beamcode;
    Kraid.immune.total_codes ++;
}

void Damage_Kraid(int p)
{
    for(int i = 0; i < Kraid.immune.total_codes; i ++)
    {
        if(Kraid.immune.codes[i] == Projectiles[p]->beamcode) return;
    }
    Kraid.stun_timer = 3;
    Kraid.health -= Projectiles[p]->damage;
    BASS_ChannelPlay(*Kraid.Sound, true);
    if(!Projectiles[p]->penetrate_enemy) Projectiles[p]->terminate = true;
    else Immunize_Kraid_From_Beam(Projectiles[p]->beamcode);
}

void Draw_Kraid(BITMAP *output)
{
    if(Kraid.active)
    {
        if(Kraid.facing_left) draw_sprite_h_flip_palette(output, Kraid.images[Kraid.to_show], Kraid.x-camera.lft, Kraid.y-camera.top, Kraid.palette);
        else draw_sprite_palette(output, Kraid.images[Kraid.to_show], Kraid.x-camera.lft, Kraid.y-camera.top, Kraid.palette);
        if(samvar.draw_collision) rect(output, Kraid.dim.lft-camera.lft, Kraid.dim.top-camera.top, Kraid.dim.rgt - camera.lft, Kraid.dim.bot - camera.top, makecol(0, 255, 0)); //Clipping
        Draw_Kraid_Support(output);
    }
}

void Tear_Kraid_Into_Giblets(int xstep, int ystep)
{
    int p;
    for(int x = 0; x < Kraid.images[Kraid.to_show]->xsize; x += xstep)
    {
        for(int y = 0; y < Kraid.images[Kraid.to_show]->ysize; y += ystep)
        {
            if(Check_If_Valid_Giblet(Kraid.images[Kraid.to_show], x, y, x+xstep, y+ystep))
            {
                Create_Kraid_Gib(Kraid.images[Kraid.to_show], x, y, x+xstep, y+ystep);
                p = Create_Sprite(Kraid.x+x, Kraid.y+y, Kraid.x+x+xstep, Kraid.y+y+ystep, 0, 1, -1);
                Imagize_Sprite_Giblet(p, Kraid_Gibs[total_kraid_gibs-1]);
            }
        }
    }
}

void Create_Kraid_Gib(CUSTOM_IMAGE *src, int lft, int top, int rgt, int bot)
{
    if(lft < 0) lft = 0;
    if(top < 0) top = 0;
    if(rgt > src->xsize) rgt = src->xsize;
    if(bot > src->ysize) bot = src->ysize;
    Kraid_Gibs = (CUSTOM_IMAGE **)realloc(Kraid_Gibs, sizeof(CUSTOM_IMAGE*) * (total_kraid_gibs+1));
    Kraid_Gibs[total_kraid_gibs] = Create_Copy_Of_Custom_Image(src, lft, top, rgt, bot);
    total_kraid_gibs ++;
}

void Update_Kraid()
{
    if(Kraid.active)
    {
        if(Kraid.health < 1)
        {
            Tear_Kraid_Into_Giblets(2, 2);
            Destroy_Kraid();
            if(!Kraid.fake)
                {
                Cut_All_Audio_Switches();
                kill_audio = true;
                BASS_ChannelPlay(Audio.Get_Item_Music, true);
                item_lockout_timer = 300;
                samvar.missiles += 75;
                samvar.missiles_max += 75;
            }
            return;
        }
        if(Kraid.stun_timer > 0)
        {
            Kraid.stun_timer --;
            Kraid.palette = 1;
            return;
        }
        Kraid.palette = 0;
        Kraid.to_show_timer ++;
        if(Kraid.to_show_timer > 7) Kraid.to_show_timer = 0;
        Kraid.to_show = Kraid.to_show_timer / 4;
        if(!samvar.invincible_timer AND samvar.screw_attack_active == -1 AND rect_clip(samvar.dim, Kraid.dim))
        {
            Injure_Samus(Kraid.damage);
        }

        if(samvar.dim.lft > Kraid.dim.rgt+32) //Samus is to Kraid's right
        {
            Kraid.move_left = false;
            Kraid.facing_left = false;
        }
        else if(samvar.dim.lft > Kraid.dim.rgt+16) //Between 16 and 32 to the right
        {
            Kraid.facing_left = false;
            Kraid.dont_move_timer = 1;
        }
        else if(samvar.dim.lft > Kraid.dim.rgt) //Between 0 and 16 to the right
        {
            Kraid.facing_left = false;
            Kraid.move_left = true;
        }
        else if(samvar.dim.rgt < Kraid.dim.lft-32) //Samus is 32 to Kraid's left
        {
            Kraid.move_left = true;
            Kraid.facing_left = true;
        }
        else if(samvar.dim.rgt < Kraid.dim.lft-16) //Between 16 and 32 to the left
        {
            Kraid.facing_left = true;
            Kraid.dont_move_timer = 1;
        }
        else if(samvar.dim.rgt < Kraid.dim.lft) //Between 0 and 16 to the left 
        {
            Kraid.facing_left = true;
            Kraid.move_left = false;
        }

        if(Kraid.dont_move_timer > 0)
        {
            Kraid.dont_move_timer --;
        }
        else
        {
            Kraid.timer ++;
            if(Kraid.timer % 2 == 0)
            {
                if(Kraid.move_left)
                {
                    if(!(Kraid.dim.lft < 48)) Kraid.x --;
                }
                else
                {
                    if(!(Kraid.dim.rgt > 192))Kraid.x ++;
                }
            }
        }

        Kraid.dim.lft = Kraid.x;
        Kraid.dim.rgt = Kraid.x + 24;
        Kraid.dim.top = Kraid.y;
        Kraid.dim.bot = Kraid.y + 32;

        int tfacing = RIGHT;
        if(Kraid.facing_left) tfacing = LEFT;

        Kraid.top_needle_timer --;
        if(Kraid.top_needle_timer < 1)
        {
            Create_Kraid_Needle(Kraid.dim.lft, Kraid.dim.top+3, tfacing);
            Kraid.top_needle_timer = rnd(120, 120);
        }
        Kraid.mid_needle_timer --;
        if(Kraid.mid_needle_timer < 1)
        {
            Create_Kraid_Needle(Kraid.dim.lft, Kraid.dim.top+12, tfacing);
            Kraid.mid_needle_timer = rnd(120, 120);
        }
        Kraid.bot_needle_timer --;
        if(Kraid.bot_needle_timer < 1)
        {
            Create_Kraid_Needle(Kraid.dim.lft, Kraid.dim.top+21, tfacing);
            Kraid.bot_needle_timer = rnd(120, 120);
        }

        Kraid.l_hook_timer --;
        if(Kraid.l_hook_timer < 1)
        {
            Create_Kraid_Hook(Kraid.dim.lft, Kraid.dim.top+3, tfacing);
            Kraid.l_hook_timer = rnd(120, 120);
        }
        Kraid.r_hook_timer --;
        if(Kraid.r_hook_timer < 1)
        {
            Create_Kraid_Hook(Kraid.dim.lft, Kraid.dim.top+2, tfacing);
            Kraid.r_hook_timer = rnd(120, 120);
        }
    }
}
