#include <allegro.h>
#include <bass.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#ifndef PANDORA
#include <conio.h>
#endif
#include <math.h>
#include <time.h>
#include <errno.h>
#include <signal.h>
#ifdef NOMW
#include "memwatch.h"
#endif

#ifdef PANDORA
#define SOUND "Sound/"
#define IMAGES "Images/"
#define SCRIPTS "Scripts/"
#define SAVES "Saves/"
#define IMAGESMAPS "Images/Maps/"
#define ROOMS "Rooms/"
#else
#define SOUND "Sound\\"
#define IMAGES "Images\\"
#define SCRIPTS "Scripts\\"
#define SAVES "Saves\\"
#define IMAGESMAPS "Images\\Maps\\"
#define ROOMS "Rooms\\"
#endif


#include "Structures.h"
#include "1Enemy_Structures.h"

#define AND &&
#define OR ||
#define PI 3.1415926

#define RIGHT 0
#define LEFT 1
#define UP 2
#define DOWN 3
#define FREEZE 10

//Clipping bits
#define STOP_NONE 0x00
#define STOP_SAMUS 0x01
#define STOP_ENEMY 0x02
#define STOP_PROJECTILE 0x04
#define STOP_SPRITE 0x08
#define STOP_FREEZE 0x10
#define STOP_ALL 0xFF

//Projectile bits
#define BEAM_DAMAGE 0x00000001  //Can be damaged by a beam
#define BEAM_ICE 0x00000002     //Can be frozen
#define BEAM_MISSILE 0x00000004 //Can be damaged by missile fire
#define BEAM_BURN 0x00000008    //Can be set on fire
#define BEAM_SHOCK 0x00000010   //Can be stunned
#define BEAM_PLASMA 0x00000020  //Can be damaged by the plasma beam
#define BEAM_CHARGE 0x00000040  //Can be damaged by charged shots
#define BEAM_BOMB 0x00000080    //Bombs will injure it
#define BEAM_SCREW 0x00000100   //Screw attack will injure it
#define BEAM_ALL 0xFFFFFFFF

//Target subtypes
#define TARGET_DOOR 0
#define TARGET_ENEMY 1
#define TARGET_RIDLEY 2
#define TARGET_KRAID 3
#define TARGET_ZEBETITE 4
//#define TARGET_RINKA 5
#define TARGET_KRAID_HOOK 6
#define TARGET_KRAID_NEEDLE 7
#define TARGET_MOTHER_BRAIN 8
#define TARGET_RINKA 9

//Enemy subtypes
#define GEEMER 0
#define SHRIEKBAT 1
#define RIPPER 2
#define REO 3
#define WAVER 4
#define ZEB 5
#define METROID 20

#define SMALLTYPE 0
#define LARGETYPE 1

#ifdef PANDORA
// needs to write some functions
char *itoa(long n, char * buff, int base)
{
	if (base==10) {
		snprintf(buff, 50, "%ld", n);
	} else { // else if hex...
		snprintf(buff, 50, "%lX", n);
	}
    return   buff;
}
#endif

int game_version = 1;
bool debug_mode = false;
bool debug_write = false;

int cnt = 0;

/////////////////////////
// Airman stuff        //
/////////////////////////
MCI_FILE *Megaman_MCI = NULL;
int megaman_mode = 0;
int megaman_tiles = 0;

/////////////////////////
// Scripts             //
/////////////////////////
bool script_mode = false;

/////////////////////////
// Audio               //
/////////////////////////
bool escape_audio = false; //If true, always use Escape_Music.
int halt_timer = 0;
bool kill_audio = false;
bool first_level_audio_switch = true;
bool play_footstep_sound = false;
bool has_played_footstep = false;
struct
{
    HSTREAM Intro_Music;
    HSTREAM Brinstar_Music;
    HSTREAM Norfair_Music;
    HSTREAM Ridley_Music;
    HSTREAM Red_Brinstar_Music;
    HSTREAM Kraid_Music;
    HSTREAM Tourian_Music;
    HSTREAM Item_Music;
    HSTREAM Get_Item_Music;
    HSTREAM Escape_Music;

    HSTREAM Acid;
    HSTREAM Samus_Hurt;
    HSTREAM Normal_Beam;
    HSTREAM Long_Beam;
    HSTREAM Ice_Beam;
    HSTREAM Wave_Beam;
    HSTREAM Spazer_Beam;
    HSTREAM Injured_Small;
    HSTREAM Injured_Large;
    HSTREAM Injured_Boss;
    HSTREAM Injured_Metroid;
    HSTREAM Invuln;
    HSTREAM Jump;
    HSTREAM Loading;
    HSTREAM Get_Missile;
    HSTREAM Get_Energy;
    HSTREAM Door_Open;
    HSTREAM Drop_Bomb;
    HSTREAM Bomb_Explode;
    HSTREAM Missile_Fire;
    HSTREAM Footstep;
    HSTREAM Morph;
    HSTREAM Ending_Music;
}Audio;

HSTREAM *Cur_Music;
HSTREAM *New_Music;

int music_fade_out_timer, music_fade_in_timer;
bool music_fade_out, music_fade_in;
bool music_fading_process;

bool quit = false;
int Bps = 60;
int CurBps = 60;

/////////////////////////
// Transition Notes    //
/////////////////////////
// Notes
// 0 = Not transitioning
// 1 = Fade out of old room
// 2 = Scroll door over
// 3 = Fade in new room

int transition_stage = 0;
int transition_counter = 0;
char transition_name[80];
char transition_to[80];
int trans_y_dif = 0;
int transition_direction;

bool transition_by_elevator;
bool elevator_clear_level;
bool elevator_swap_music = false;
int elevator_trans_cooldown = 0;

BITMAP *buffer;
BITMAP *transition_buffer_old;
BITMAP *transition_buffer_new;
BITMAP *black_buf_old;
BITMAP *black_buf_new;

//////////////////////
// Intro            //
//////////////////////
// Notes
// 0 = Developer's stuff
// 1 = Fading in
// 2 = METROID fades in
// 3 = Menu
// 4 = New Game menu
// 40 = Transitioning to New Game menu
// 5 = Load Game menu

bool intro_running = true;
bool init_intro = true;
int intro_stage = 0;
int intro_counter = 0;
int star_counter = 0;
bool first_time_intro = true;
int intro_show_message_timer = 0;

int intro_cursor = 0;
int intro_press = 0;

int intro_en_speed = 1;
int intro_en_health = 1;

char **intro_file_names = NULL;
int intro_files_found = 0;

char intro_new_game_name[80];
int edit_loc = 0;

BITMAP *intro_black;
BITMAP *intro_baseline; //These are scrapped when the intro is made
BITMAP *intro_metroid;
BITMAP *intro_metroid_mask;
BITMAP *intro_new;
BITMAP *load;
BITMAP *arrow;
BITMAP *intro_backup;

PIXEL *Stars;
int total_stars = 0;

int boxcol_one;
int boxcol_two;
BITMAP *box_corner;

//////////////////////
// Escape Sequence  //
//////////////////////
bool escape_active = false;
int countdown_timer = 1000;
int countdown_sub_timer = 60;
int brain_remove_timer = 120;

//////////////////////
// Game Over stuff  //
//////////////////////
int game_over_fade_pause = 50;
BITMAP *game_over_fade_image = NULL;
BITMAP *game_over_black_buf = NULL;
bool game_over_mode = false;
int game_over_fade = 255;
int sam_screen_x, sam_screen_y;
bool show_game_over_text = false;

//////////////////////
// Credits          //
//////////////////////
bool credits_running = false;
bool init_credits = false;
int credits_stage = 0;  //0 = Congrats text, 5 = Scrolling credits
int credits_timer;
bool skip_drawing_credits_this_pulse = false;
BITMAP *credits_back;
BITMAP *credits_text;

//////////////////////
// Pausing          //
//////////////////////
bool game_paused = false;
int pause_cursor = 0;
bool newpause = false;
int press_cooldown = 0;

//////////////////////
// Bosses           //
//////////////////////

RIDLEY Ridley;
CUSTOM_IMAGE **Ridley_Gibs;
CUSTOM_IMAGE *Ridley_Fireball;
int total_ridley_gibs = 0;

KRAID Kraid;
CUSTOM_IMAGE **Kraid_Gibs;
int total_kraid_gibs = 0;
KRAID_NEEDLE *Needles;
int total_needles = 0;
KRAID_HOOK *Hooks;
int total_hooks = 0;
int kraid_hook_vulnerabilities = 0;

//////////////////////
// Datafiles        //
//////////////////////

DATAFILE *Image_Datafile = NULL;

//////////////////////
// Custom           //
//////////////////////

CUSTOM_IMAGE *Custom_Images = NULL;
int total_isf_tiles;
int total_ipf_tiles;

//////////////////////
// HUD Stuff        //
//////////////////////

CUSTOM_IMAGE *en_display;
int vertical_scroll_dif;
int to_reach_vert;

//////////////////////
// Color Variables  //
//////////////////////

int transparent;
int white;
int black = 0;
int cur_palette = 0;

/////////////////////
// Samus Variables //
/////////////////////
int frame = 0;

typedef struct
{
    bool long_beam, charge_beam, ice_beam, wave_beam, spazer_beam, plasma_beam, burn_beam, helix_beam, shock_beam;
    bool varia_suit, gravity_suit, corrosion_suit;
    bool morph_ball, bombs, spring_ball, screw_attack;
    bool high_jump, space_jump, speed_booster;
}INVENTORY;

typedef struct
{
    bool penetrate_enemy;
    bool penetrate_world;
    bool freeze;
    bool burn;
    bool shock;
    int duration;
    int damage;
    HSTREAM *Sound;
    CUSTOM_IMAGE *image;
    int palette;
    int width, height;
    float xspeed, yspeed;
    int facing;
    int origin_x, origin_y;
    unsigned char properties;
    int create_flag; //0 is normal.  1 is 3 beams (Spazer) and 2 is 2 beams (Helix)
    int wave_flag;
    int refire_rate;
    int img_offset_x, img_offset_y;
    int not_less_than_pos;
    int start_timer;
    DIMENSION clip_dif;
}BEAMDATA;

struct
{
    //Stats, like HP/Missiles
    int energy, energy_max;
    int missiles, missiles_max;

    //Clipping and Position
    int x, y;
    float xremainder, yremainder;
    float xspeed, yspeed, xaccel, yaccel;
    bool hamper_movement; //True when in acid/water or when a Metroid is latched on.
    bool is_in_acid;
    int acid_damage_timer; //Counts to 0 from 30.  Always 60 when not wet.  Varia sets it to 60 on reset instead of 30.
    DIMENSION dim;
    bool touch_bot, touch_lft, touch_rgt, touch_top;
    bool ball_mode;
    bool crouch_mode;
    bool crouch_release_flag;
    int ball_frame;
    int boost_timer;
    int falling_lockout;
    bool began_jump;
    int cur_clip_detect;

    //Controls and Drawing
    int press_left, press_right, press_jump, press_fire, press_pause, press_up, press_down, press_debug, release_fire;
    int press_switchitem, release_switchitem, press_start;
    int press_lshoulder, press_rshoulder;
    int press_saveout;
    int release_jump;
    bool rejump;
    int frame, facing, run_frameup, flip, aim_up, falling, spin_jump, jump, jump_cnt, cant_jump, fall_cnt, runframe;
    int launch_height, shot_cnt;
    int invincible_timer;
    bool lockout_controls;
    bool disable_spin_draw;
    int lockout_timer;
    bool missile_mode;
    int palette;
    int qpalette;
    bool is_in_water;
    int screw_attack_active;

    bool ridley_dead, kraid_dead;

    //Inventory
    char cur_beam_type;
    bool shot_reverse;
    int cur_beam_damage;
    INVENTORY Using;
    INVENTORY Collected;
    BEAMDATA Beam_Data;

    //Debug
    bool draw_clips, draw_collision;
    int draw_debug;
}samvar;
CUSTOM_IMAGE *Sam_Images[18];
SAMUS_FRAMES Sam_Frame;
int sam_img_offset_x[18];
int sam_img_offset_y[18];
CUSTOM_IMAGE *Sam_Display = NULL;
int sam_offset_x, sam_offset_y;
int draw_val = 0;
bool frame_flip = false;

bool random_game = false;
bool item_toggle = false;
bool item_toggle_repress = false;

bool do_not_draw_samus = false;

DIMENSION camera;
DIMENSION camera_max;
DIMENSION camera_scroll;
DIMENSION camera_save;
int bg_pos_save_x = 0;
int bg_start_save_x = 0;
int bg_pos_save_y = 0;
int bg_start_save_y = 0;

RECOLOR_PACK *Samus_Recolor;

int curlev = -1;

float gravity = 0.050;

////////////////////////
// Subscreen          //
////////////////////////

bool subscreen_up = false;
int subscreen_page = 0;
int subscreen_cooldown = 120;
bool exiting_subscreen = false;
int subscreen_fade_stage = 0;
int subscreen_ctr = 0;
BITMAP *subscreen_reserve_buffer = NULL;
BITMAP *subscreen_black_buffer = NULL;
bool draw_to_reserve_buffer_this_pulse = false;

BITMAP *Map_Base = NULL;
BITMAP *Item_Base = NULL;
BITMAP *Plain_Suit = NULL;
BITMAP *Varia_Suit = NULL;
BITMAP *Red_Button, *Pink_Button, *Gray_Button;
FONT *SubscreenFont = NULL;
SUBSCREEN_ITEM Subscreen_Item[18];
int subscreen_item_selected = 0;
int cursor_ctr = 0;
bool cursor_ctr_up = true;
int subscreen_press_cooldown = 0;
int column_pos_left = 0;
int column_pos_right = 5;
bool using_left_column = true;

////////////////////////
// Saving             //
////////////////////////
SAVEFILE *Saveout = NULL;
char *curlev_name = NULL;
int curlev_len = 0;
char *Saveout_path = NULL;

SAVEZONE Savezone;

////////////////////////
// Items              //
////////////////////////

MCI_FILE *Master_MCI;

ITEM *Items = NULL;
int total_items = 0;

CUSTOM_IMAGE *Item_ensmall[2];
CUSTOM_IMAGE *Item_missile[2];

CUSTOM_IMAGE *Item_missiletank[2];
CUSTOM_IMAGE *Item_energytank[2];
CUSTOM_IMAGE *Item_longbeam[2];

BITMAP *Fanfare_Item[38];

int item_lockout_timer = 0;

////////////////////////
// Shooting Variables //
////////////////////////

CUSTOM_IMAGE *Shot_normal = NULL;
CUSTOM_IMAGE *Shot_spazer = NULL;
CUSTOM_IMAGE *Shot_plasma = NULL;
CUSTOM_IMAGE *Shot_helix = NULL;
CUSTOM_IMAGE *Shot_helix_trail = NULL;
CUSTOM_IMAGE *Shot_missile = NULL;
CUSTOM_IMAGE *Shot_wave = NULL;
CUSTOM_IMAGE *Shot_burn = NULL;
CUSTOM_IMAGE *Shot_bomb[2];
CUSTOM_IMAGE *Missile_Explode = NULL;
CUSTOM_IMAGE *Explosion_Bit;
PROJECTILE **Projectiles = NULL;
int global_projectiles_fired = 0;
int total_projectiles = 0;
float projectile_lookup_table[36];

CUSTOM_IMAGE *Beam_To_Display = Shot_normal;

TARGET **Targets = NULL;
int total_targets = 0;

////////////////////////
// Sprites            //
////////////////////////

SPRITE **Sprite_List = NULL;
int total_sprites = 0;

SPRITE_SPAWNER **Sprite_Spawner = NULL;
int total_sprite_spawners = 0;

CUSTOM_IMAGE *Explosion_Images[2];
CUSTOM_IMAGE *Droplet_Images[2];

//////////////////////
// Campaign         //
//////////////////////

CAMPAIGN *Campaign = NULL;
bool campaign_mode = false;

//////////////////////
// Room Variables   //
//////////////////////

ROOM *Room = NULL; //The room we're currently in
ROOM *Load_Room; //The room we're transitioning to
BITMAP *Background = NULL;
bool using_background = false;
int bg_start_x, bg_start_y;
int bg_scroll_factor_x, bg_scroll_factor_y;
int bg_scroll_pos_x, bg_scroll_pos_y;

CUSTOM_IMAGE **Tiles = NULL;
//BITMAP **Tiles = NULL;//Images of the tiles in the rooms

unsigned char **cliparray = NULL;//The actual physical clipping of the room

#define NORMAL_DOOR 0
#define MISSILE_DOOR 1
DOOR *Doors = NULL;
int total_doors = 0;
CUSTOM_IMAGE *Door_Images[4];
int doorteam = 2;
int door_vuln = BEAM_ALL;
int missile_door_vuln = BEAM_MISSILE;

ELEVATOR *Elevators = NULL;
int total_elevators = 0;
CUSTOM_IMAGE *Elevator_img = NULL;

TRANSITION *Transitions = NULL;
int total_transitions = 0;
int door_distance_bot, door_distance_top;
int exit_distance_bot, exit_distance_top;

BREAKABLE *Breakables = NULL;
int total_breakables = 0;
CUSTOM_IMAGE *Break_Images[2];

SAMSTART Samstart;
bool use_Samstart = false;
bool use_Samstart_this_push = false;

WATER_ZONE *Water;
int total_water_zones = 0;
int water_array[6];
int water_global_timer = 0;
BITMAP *Water_Img_Top = NULL;
BITMAP *Water_Img_Mid = NULL;
BITMAP *Acid_Img_Top = NULL;
BITMAP *Acid_Img_Mid = NULL;
CUSTOM_IMAGE *Water_Splash[8];
#define WSIZE 128

int water_ent_a_lft, water_ent_a_top, water_ent_a_rgt, water_ent_a_bot;
int water_ent_b_lft, water_ent_b_top, water_ent_b_rgt, water_ent_b_bot;
int water_ent_c_lft, water_ent_c_top, water_ent_c_rgt, water_ent_c_bot;

//////////////////////
// Map              //
//////////////////////

int cur_map_pos_x = 2;
int cur_map_pos_y = 2;
int total_maps;
MAP **Maps = NULL;
MAP *Curmap = NULL;
int map_timer = 0;
RECOLOR_PACK *Map_Recolor;
BITMAP *map_cursor = NULL;
short cursor_array[] = {0, 1, 2, 2, 2, 2, 2, 1, 0};

//////////////////////
// Camera Bounding  //
//////////////////////

CAMBOUND *Cambounds = NULL;
int total_cambounds = 0;

#define BOUND_LFT 0x0008
#define BOUND_TOP 0x0001
#define BOUND_RGT 0x0002
#define BOUND_BOT 0x0004

//////////////////////////////////
// Stuff related to keypressing //
//////////////////////////////////
struct
{
    int right, left, up, down, jump, fire, start, switchitem;
    int lshoulder, rshoulder;
    int cancel, speed, slow, quit;
    int saveout;
    int frame_plus, frame_minus, pix_rgt, pix_lft, pix_bot, pix_top, screen_shot;
    int clip_draw, debug_draw, collision_draw;
}keys;

struct
{
    int right, left, up, down, jump, fire;
    int lshoulder, rshoulder;
    int select, start, run;
}key_presses;

KEYLIST *Keylist;
int total_keys;
CONTROLLIST *Controls;
int total_controls = 0;

///////////////////////////////////
// Enemies                       //
///////////////////////////////////

CUSTOM_IMAGE *Enemy_Images[43];
//BITMAP *Enemy_Images[20];
ENEMY **Enemies;
int total_enemies = 0;
ENEMY_SPAWNER *Enemy_Spawner;
int total_enemy_spawners = 0;
RINKA *Rinka;
int total_rinkas = 0;
int metroid_latch_index = -1;
int metroid_latch_count = 0; //When it counts to 0 from 60, you lose 1 energy.

ZEBETITE *Zebetite;
int total_zebetites = 0;
CUSTOM_IMAGE *Zebetite_Img[4];
int zebetite_vulnerabilities = BEAM_MISSILE;

int vulnerable_all = BEAM_ALL;
int dummy_stun_timer;

MINIGUN *Minigun;
int total_miniguns = 0;

int enemy_team = 1;

CUSTOM_IMAGE *Minigun_H;
CUSTOM_IMAGE *Minigun_V;
CUSTOM_IMAGE *Minigun_D;
CUSTOM_IMAGE *Minigun_Shot_V;
CUSTOM_IMAGE *Minigun_Shot_D;
CUSTOM_IMAGE *Rinka_img;

bool rinka_flag = false; //If true, the next enemy spawner is a rinka spawner

///////////////////////////////////
//  Client Sided Variables       //
///////////////////////////////////
CVAR *CVars = NULL;
int total_cvars = 0;
int screen_var = 0;

////////////////////////
// Speed Counter      //
////////////////////////
volatile long speed_counter = 0;
void increment_speed_counter()
{
    speed_counter++;
}
