////////////////////////////////////////////////////////////////////////////////
//  Metroids
//
//  Metroid Substructure
//
//  0 - Latch state.  0 if unlatched, 1 if latched.
//  1 - Bombed off health.  When latched, 5 bombs will unlatch.
//  2 - Knockback state.  When unlatched, Metroid flies away for 30 frames.
//  3 - Xspeed.  Divide by 10 for actual speed.  Subtract 30 (it can be -3).
//  4 - Yspeed.  Divide by 10 for actual speed.  Subtract 30 (it can be -3).
//  5 - Friction flag.  Alternates between 0 and 1.  Metroids can accelerate when it is 0 only.
////////////////////////////////////////////////////////////////////////////////
int Create_Metroid(int x, int y, bool hardtype)
{
    for(int i = 0; i < total_enemies; i ++) //Scan the old enemies to see if one is unused
    {
        if(!Enemies[i]->active)
        {
            Place_Metroid_In_Slot(i, x, y, hardtype);
            return i;
        }
    }
    Enemies = (ENEMY **)realloc(Enemies, sizeof(ENEMY *) * (total_enemies + 1));
    Enemies[total_enemies] = (ENEMY *)malloc(sizeof(ENEMY));
    Enemies[total_enemies]->substructure = NULL;
    Enemies[total_enemies]->display.images = NULL;
    Enemies[total_enemies]->immune.codes = NULL;

    Place_Metroid_In_Slot(total_enemies, x, y, hardtype);
    total_enemies ++;
    return total_enemies - 1;
}

void Place_Metroid_In_Slot(int i, int x, int y, bool hardtype)
{
    Enemies[i]->active = true;
    Enemies[i]->type_flag = METROID;
    Enemies[i]->coord.x = x;
    Enemies[i]->coord.y = y;
    Enemies[i]->coord.orientation = RIGHT;
    Enemies[i]->bound.lft = x;
    Enemies[i]->bound.top = y;
    Enemies[i]->bound.rgt = x + 24;
    Enemies[i]->bound.bot = y + 23;
    Enemies[i]->stats.health = 80;
    Enemies[i]->stats.damage = 0;
    Enemies[i]->stats.speed_modifier = 1;
    Enemies[i]->stats.vulnerabilities = BEAM_ICE;
    Enemies[i]->stats.hardtype = hardtype;
    if(Enemies[i]->display.images) free(Enemies[i]->display.images);
    Enemies[i]->display.images = (CUSTOM_IMAGE **)malloc(sizeof(CUSTOM_IMAGE *) * 2);
    Enemies[i]->display.images[0] = Enemy_Images[41];
    Enemies[i]->display.images[1] = Enemy_Images[42];
    Enemies[i]->display.hflip = false;
    Enemies[i]->display.vflip = false;
    Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->display.rotation = 0;
    Enemies[i]->display.to_show = 0;
    Enemies[i]->display.update = &Update_Metroid_GFX;
    Enemies[i]->funcptr = &Update_Metroid;
    Enemies[i]->size_of_substructure = 6;
    Enemies[i]->substructure = (unsigned char*)malloc(sizeof(unsigned char) * Enemies[i]->size_of_substructure);
    Enemies[i]->substructure[0] = 0;
    Enemies[i]->substructure[1] = 0;
    Enemies[i]->substructure[2] = 0;
    Enemies[i]->substructure[3] = 100;
    Enemies[i]->substructure[4] = 100;
    Enemies[i]->substructure[5] = 0;
    Enemies[i]->team = 1;
    Enemies[i]->stun_timer = 0;
    Enemies[i]->freeze_timer = 0;
    Enemies[i]->palette = 0;
    Enemies[i]->refreeze_timer = 0;
    Enemies[i]->Sound = &Audio.Injured_Metroid;
    Add_To_Target_List(TARGET_ENEMY, i);

    if(Enemies[i]->immune.codes) free(Enemies[i]->immune.codes);
    Enemies[i]->immune.codes = NULL;
    Enemies[i]->immune.total_codes = 0;
}

void Update_Metroid_GFX(int i)
{
    Enemies[i]->display.draw_cnt ++;
    if(Enemies[i]->display.draw_cnt > 9) Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->display.to_show = Enemies[i]->display.draw_cnt / 5;
}

void Update_Metroid(int i)
{
    if(Enemies[i]->stats.health <= 0) //Metroid is down, and he's not getting up! :(
    {
        Create_Random_Item(Enemies[i]->bound, LARGETYPE, true);
        int spritenum = Create_Sprite(((Enemies[i]->bound.lft+Enemies[i]->bound.rgt)/2)-16, ((Enemies[i]->bound.top+Enemies[i]->bound.bot)/2)-16, 3, 2, 6);
        if(spritenum != -1) Imagize_Sprite_Explosion(spritenum);
        Enemies[i]->active = false;
        Remove_From_Target_List(&Enemies[i]->stats.health);
        Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        return;
    }
    if(Enemies[i]->stun_timer > 0)
    {
        Enemies[i]->stun_timer --;
        return;
    }
    if(Enemies[i]->freeze_timer > 0)
    {
        Enemies[i]->stats.vulnerabilities = BEAM_ICE + BEAM_MISSILE;
        Freeze_Clip_Area(Enemies[i]->bound, STOP_FREEZE);
        Enemies[i]->freeze_timer --;
        if(Enemies[i]->freeze_timer == 0) Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        if(Enemies[i]->freeze_timer < 50)
        {
            if((Enemies[i]->freeze_timer % 4) < 2)
            {
                Enemies[i]->palette = 2;
            }
            else
            {
                if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
                else Enemies[i]->palette = 0;
            }
        }
        else
        {
            Enemies[i]->palette = 2;
        }
        return;
    }
    Enemies[i]->stats.vulnerabilities = BEAM_ICE;
    if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
    else Enemies[i]->palette = 0;

    if(Enemies[i]->substructure[5] == 0) Enemies[i]->substructure[5] == 1;
    else Enemies[i]->substructure[5] = 0;

    if(Enemies[i]->substructure[0] == 0 AND Enemies[i]->substructure[5] == 0) //Unlatched.  Determine where Samus is and move towards her.
    {
        if(Enemies[i]->bound.lft+12 < samvar.dim.lft+3) //Accelerate right
        {
            Enemies[i]->substructure[3] ++;
            if(Enemies[i]->substructure[3] > 130) Enemies[i]->substructure[3] = 130;
        }
        else if(Enemies[i]->bound.lft+12 > samvar.dim.lft+3) //Accelerate left
        {
            Enemies[i]->substructure[3] --;
            if(Enemies[i]->substructure[3] < 70) Enemies[i]->substructure[3] = 70;
        }
        if(Enemies[i]->bound.top+12 < samvar.dim.top+3) //Accelerate down
        {
            Enemies[i]->substructure[4] ++;
            if(Enemies[i]->substructure[4] > 130) Enemies[i]->substructure[4] = 130;
        }
        else if(Enemies[i]->bound.top+12 > samvar.dim.top+3) //Accelerate up
        {
            Enemies[i]->substructure[4] --;
            if(Enemies[i]->substructure[4] < 70) Enemies[i]->substructure[4] = 70;
        }
        //printf("Xspeed %f Yspeed %f\n", (Enemies[i]->substructure[3]-100)/20.0, (Enemies[i]->substructure[4]-100)/20.0);
    }

    if(Enemies[i]->substructure[2] > 0) //Movement is reversed as the Metroid stumbles off.
    {
        Enemies[i]->substructure[2] --;
        if(Enemies[i]->substructure[3] <  90) Enemies[i]->substructure[3] = 90;
        if(Enemies[i]->substructure[4] >  90) Enemies[i]->substructure[4] = 90;
        if(Enemies[i]->substructure[3] > 110) Enemies[i]->substructure[3] = 110;
        if(Enemies[i]->substructure[4] > 110) Enemies[i]->substructure[4] = 110;
        Move_Enemy(i, -((Enemies[i]->substructure[3]-100)/10), ((Enemies[i]->substructure[4]-100)/10));
    }
    else
    {
        Move_Enemy(i, (Enemies[i]->substructure[3]-100)/10, (Enemies[i]->substructure[4]-100)/10);
    }

    if(metroid_latch_index == -1 AND Enemies[i]->substructure[2] == 0)
    {
        //Check if the Metroid can latch on to Samus
        if(rect_clip(samvar.dim, Enemies[i]->bound))
        {
            metroid_latch_index = i;
            Enemies[i]->substructure[0] = 1;
            Enemies[i]->substructure[1] = 5;
        }
        else
        {
            Enemies[i]->substructure[0] = 0;
        }
    }
    else if(metroid_latch_index == i)
    {
        Enemies[i]->coord.x = samvar.dim.lft - 8;
        Enemies[i]->coord.y = samvar.y + 0;
    }
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->bound.lft+24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->bound.top+23;

    Enemies[i]->display.update(i);
}
