RECOLOR_PACK *Create_Recolor(int uniques, ...)
{
    va_list arglist;
    va_start(arglist, uniques);

    RECOLOR_PACK *recol = (RECOLOR_PACK *)malloc(sizeof(RECOLOR_PACK));
    recol->palettes = 1;
    recol->uniques = uniques;
    recol->cols = (int **)malloc(sizeof(int *) * 1);
    recol->cols[0] = (int *)malloc(sizeof(int) * uniques);
    for(int i = 0; i < uniques; i ++)
    {
        recol->cols[0][i] = va_arg(arglist, int);
    }

    va_end(arglist);
    return recol;
}

void Add_Palette(RECOLOR_PACK **recol, ...)
{
    va_list arglist;
    va_start(arglist, recol);
    
    (*recol)->palettes ++;
    (*recol)->cols = (int **)realloc((*recol)->cols, sizeof(int *) * (*recol)->palettes);
    (*recol)->cols[(*recol)->palettes-1] = (int *)malloc(sizeof(int) * (*recol)->uniques);
    for(int i = 0; i < (*recol)->uniques; i ++)
    {
        (*recol)->cols[(*recol)->palettes-1][i] = va_arg(arglist, int);
    }

    va_end(arglist);
}

void Recolor_Blit(BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height, RECOLOR_PACK *recol, int palette)
{
    if(palette == 0)
    {
        blit(source, dest, source_x, source_y, dest_x, dest_y, width, height);
    }
    bool failflag;
    int pixval;
    for(int x = source_x; x < source_x + width; x ++)
    {
        for(int y = source_y; y < source_y + height; y ++)
        {
            pixval = getpixel(source, x, y);
            failflag = false;
            for(int i = 0; i < recol->uniques; i ++)
            {
                if(pixval == recol->cols[0][i])
                {
                    putpixel(dest, dest_x+x-source_x, dest_y+y-source_y, recol->cols[palette][i]);
                    i = recol->uniques;
                    failflag = true;
                }
            }
            if(!failflag AND pixval != transparent) 
            putpixel(dest, dest_x+x-source_x, dest_y+y-source_y, pixval);
        }
    }
}


void Recolor_Bitmap(BITMAP *dest, BITMAP *source, int xt, int yt, RECOLOR_PACK *recol, int palette)
{
    if(palette == 0)
    {
        draw_sprite(dest, source, xt, yt);
        return;
    }
    bool failflag;
    int pixval;
    for(int x = 0; x < source->w; x ++)
    {
        for(int y = 0; y < source->h; y ++)
        {
            failflag = false;
            pixval = getpixel(source, x, y);
            for(int i = 0; i < recol->uniques; i ++)
            {
                if(pixval == recol->cols[0][i])
                {
                    putpixel(dest, xt+x, yt+y, recol->cols[palette][i]);
                    i = recol->uniques;
                    failflag = true;
                }
            }
            if(!failflag AND pixval != transparent) putpixel(dest, xt+x, yt+y, pixval);
        }
    }
}

void Recolor_Bitmap_h_flip(BITMAP *dest, BITMAP *source, int xt, int yt, RECOLOR_PACK *recol, int palette)
{
    if(palette == 0)
    {
        draw_sprite_h_flip(dest, source, xt, yt);
        return;
    }
    bool failflag;
    int pixval;
    for(int x = 0; x < source->w; x ++)
    {
        for(int y = 0; y < source->h; y ++)
        {
            failflag = false;
            pixval = getpixel(source, source->w-x-1, y);
            for(int i = 0; i < recol->uniques; i ++)
            {
                if(pixval == recol->cols[0][i])
                {
                    putpixel(dest, xt+x, yt+y, recol->cols[palette][i]);
                    i = recol->uniques;
                    failflag = true;
                }
            }
            if(!failflag AND pixval != transparent) putpixel(dest, xt+x, yt+y, pixval);
        }
    }
}

void Recolor_Bitmap_v_flip(BITMAP *dest, BITMAP *source, int xt, int yt, RECOLOR_PACK *recol, int palette)
{
    if(palette == 0)
    {
        draw_sprite_v_flip(dest, source, xt, yt);
        return;
    }
    bool failflag;
    int pixval;
    for(int x = 0; x < source->w; x ++)
    {
        for(int y = 0; y < source->h; y ++)
        {
            failflag = false;
            pixval = getpixel(source, x, source->h-y-1);
            for(int i = 0; i < recol->uniques; i ++)
            {
                if(pixval == recol->cols[0][i])
                {
                    putpixel(dest, xt+x, yt+y, recol->cols[palette][i]);
                    i = recol->uniques;
                    failflag = true;
                }
            }
            if(!failflag AND pixval != transparent) putpixel(dest, xt+x, yt+y, pixval);
        }
    }
}

void Recolor_Bitmap_vh_flip(BITMAP *dest, BITMAP *source, int xt, int yt, RECOLOR_PACK *recol, int palette)
{
    if(palette == 0)
    {
        draw_sprite_vh_flip(dest, source, xt, yt);
        return;
    }
    bool failflag;
    int pixval;
    for(int x = 0; x < source->w; x ++)
    {
        for(int y = 0; y < source->h; y ++)
        {
            failflag = false;
            pixval = getpixel(source, source->w-x-1, source->h-y-1);
            for(int i = 0; i < recol->uniques; i ++)
            {
                if(pixval == recol->cols[0][i])
                {
                    putpixel(dest, xt+x, yt+y, recol->cols[palette][i]);
                    i = recol->uniques;
                    failflag = true;
                }
            }
            if(!failflag AND pixval != transparent) putpixel(dest, xt+x, yt+y, pixval);
        }
    }
}

