////////////////////////////////////////////////////////////////////////////////
//  Reos
//
//  Reo substructure
//  0 = If this is true, the Reo has detected Samus and is diving
//  1 = If true, the Reo is going back up after a dive
//  2 = Counter for waiting to dive
//  3 = yspeed
//  4 = yaccel
//  5 = disable turnaround if 1
//
////////////////////////////////////////////////////////////////////////////////
int Create_Reo(int x, int y, bool hardtype)
{
    for(int i = 0; i < total_enemies; i ++) //Scan the old enemies to see if one is unused
    {
        if(!Enemies[i]->active)
        {
            Place_Reo_In_Slot(i, x, y, hardtype);
            return i;
        }
    }
    Enemies = (ENEMY **)realloc(Enemies, sizeof(ENEMY *) * (total_enemies + 1));
    Enemies[total_enemies] = (ENEMY *)malloc(sizeof(ENEMY));
    Enemies[total_enemies]->substructure = NULL;
    Enemies[total_enemies]->display.images = NULL;
    Enemies[total_enemies]->immune.codes = NULL;

    Place_Reo_In_Slot(total_enemies, x, y, hardtype);
    total_enemies ++;
    return total_enemies - 1;
}

void Place_Reo_In_Slot(int i, int x, int y, bool hardtype)
{
    Enemies[i]->active = true;
    Enemies[i]->type_flag = GEEMER;
    Enemies[i]->funcptr = &Update_Reo;
    if(Enemies[i]->substructure) free(Enemies[i]->substructure);
    Enemies[i]->size_of_substructure = 6;
    Enemies[i]->substructure = (unsigned char*)malloc(sizeof(unsigned char) * Enemies[i]->size_of_substructure);
    Enemies[i]->substructure[0] = 0;
    Enemies[i]->substructure[1] = 0;
    Enemies[i]->substructure[2] = 0;
    Enemies[i]->substructure[3] = 0;
    Enemies[i]->substructure[4] = 0;
    Enemies[i]->substructure[5] = 0;
    Enemies[i]->coord.x = x;
    Enemies[i]->coord.y = y;
    Enemies[i]->coord.orientation = RIGHT;
    Enemies[i]->stats.health = 4;
    if(hardtype) Enemies[i]->stats.health *= 2;
    Enemies[i]->stats.hardtype = hardtype;
    Enemies[i]->stats.damage = 10;
    Enemies[i]->stats.speed_modifier = 1;
    Enemies[i]->stats.vulnerabilities = BEAM_ALL;
    if(Enemies[i]->display.images) free(Enemies[i]->display.images);
    Enemies[i]->display.images = (CUSTOM_IMAGE **)malloc(sizeof(CUSTOM_IMAGE *) * 2);
    Enemies[i]->display.images[0] = Enemy_Images[11];
    Enemies[i]->display.images[1] = Enemy_Images[12];
    Enemies[i]->display.to_show = 0;
    Enemies[i]->display.rotation = 0;
    Enemies[i]->display.hflip = false;
    Enemies[i]->display.vflip = false;
    Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->team = 1;
    Enemies[i]->Sound = &Audio.Injured_Small;
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 19;
    Enemies[i]->stun_timer = 0;
    Enemies[i]->freeze_timer = 0;
    Enemies[i]->refreeze_timer = 0;
    Add_To_Target_List(TARGET_ENEMY, i);

    if(Enemies[i]->immune.codes) free(Enemies[i]->immune.codes);
    Enemies[i]->immune.codes = NULL;
    Enemies[i]->immune.total_codes = 0;
}

void Update_Reo(int i)
{
    if(Enemies[i]->stats.health <= 0) //Reo got wrecked :(
    {
        Create_Random_Item(Enemies[i]->bound, SMALLTYPE, false);
        int spritenum = Create_Sprite(((Enemies[i]->bound.lft+Enemies[i]->bound.rgt)/2)-16, ((Enemies[i]->bound.top+Enemies[i]->bound.bot)/2)-16, 3, 2, 6);
        if(spritenum != -1) Imagize_Sprite_Explosion(spritenum);
        Enemies[i]->active = false;
        Remove_From_Target_List(&Enemies[i]->stats.health);
        Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        return;
    }
    if(Enemies[i]->stun_timer > 0)
    {
        Enemies[i]->stun_timer --;
        return;
    }
    if(Enemies[i]->freeze_timer > 0)
    {
        Freeze_Clip_Area(Enemies[i]->bound, STOP_FREEZE);
        Enemies[i]->freeze_timer --;
        if(Enemies[i]->freeze_timer == 0) Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        if(Enemies[i]->freeze_timer < 50)
        {
            if((Enemies[i]->freeze_timer % 4) < 2)
            {
                Enemies[i]->palette = 2;
            }
            else
            {
                if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
                else Enemies[i]->palette = 0;
            }
        }
        else
        {
            Enemies[i]->palette = 2;
        }
        return;
    }
    if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
    else Enemies[i]->palette = 0;

    Enemies[i]->display.draw_cnt ++;
    if(Enemies[i]->display.draw_cnt >= 8) Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->display.to_show = Enemies[i]->display.draw_cnt / 4;

    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 24;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 19;

    if(!Enemies[i]->substructure[0]) //Reo is waiting to dive
    {
        Enemies[i]->substructure[2] ++;
        Enemies[i]->substructure[5] = 0;
        if(Enemies[i]->substructure[2] >= 40)
        {
            Enemies[i]->substructure[2] = 0;
            if(Enemies[i]->bound.lft - 100 < samvar.dim.rgt AND Enemies[i]->bound.rgt + 100 > samvar.dim.lft)
            {
                Enemies[i]->substructure[0] = 1;
                if(samvar.dim.rgt < Enemies[i]->bound.lft) Enemies[i]->coord.orientation = LEFT;
                else if(samvar.dim.lft > Enemies[i]->bound.rgt) Enemies[i]->coord.orientation = RIGHT;
                else Enemies[i]->coord.orientation = DOWN;
            }
        }
    }
    else if(Enemies[i]->substructure[0])
    {
        if(Enemies[i]->substructure[1]) //Reo is climbing after a dive
        {
            Enemies[i]->substructure[5] = 0;
            if(Enemies[i]->substructure[3] > 0) Enemies[i]->substructure[3] --;
            if(Enemies[i]->coord.orientation == LEFT)
            {
                Move_Enemy(i, -1, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft, Enemies[i]->bound.rgt, Enemies[i]->bound.top-1, 0x02))
                {
                    Enemies[i]->substructure[0] = 0;
                    Enemies[i]->substructure[1] = 0;
                    Enemies[i]->substructure[2] = 0;
                }
                if(Scan_Clips_Y(Enemies[i]->bound.top, Enemies[i]->bound.bot, Enemies[i]->bound.lft-1, 0x02))
                {
                    Enemies[i]->coord.orientation = RIGHT;
                }
            }
            else if(Enemies[i]->coord.orientation == DOWN)
            {
                Move_Enemy(i, 0, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft, Enemies[i]->bound.rgt, Enemies[i]->bound.top-1, 0x02))
                {
                    Enemies[i]->substructure[0] = 0;
                    Enemies[i]->substructure[1] = 0;
                    Enemies[i]->substructure[2] = 0;
                }
                if(Scan_Clips_Y(Enemies[i]->bound.top, Enemies[i]->bound.bot, Enemies[i]->bound.lft-1, 0x02))
                {
                    Enemies[i]->coord.orientation = RIGHT;
                }
            }
            else if(Enemies[i]->coord.orientation == RIGHT)
            {
                Move_Enemy(i, 1, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft, Enemies[i]->bound.rgt, Enemies[i]->bound.top-1, 0x02))
                {
                    Enemies[i]->substructure[0] = 0;
                    Enemies[i]->substructure[1] = 0;
                    Enemies[i]->substructure[2] = 0;
                }
                if(Scan_Clips_Y(Enemies[i]->bound.top, Enemies[i]->bound.bot, Enemies[i]->bound.rgt+1, 0x02))
                {
                    Enemies[i]->coord.orientation = LEFT;
                }
            }
        }
        else  //Reo is diving
        {
            if(Enemies[i]->substructure[3] < 8) Enemies[i]->substructure[3] ++;
            if(Enemies[i]->coord.orientation == LEFT)
            {
                Move_Enemy(i, -1, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft+3, Enemies[i]->bound.rgt-3, Enemies[i]->bound.bot+1, 0x02))
                {
                    Enemies[i]->substructure[5] = 1;
                }
                if(Scan_Clips_Y(Enemies[i]->bound.top, Enemies[i]->bound.bot, Enemies[i]->bound.lft-1, 0x02))
                {
                    Enemies[i]->coord.orientation = RIGHT;
                    Enemies[i]->substructure[5] = 1;
                }
                if(samvar.dim.lft > Enemies[i]->bound.rgt AND Enemies[i]->substructure[5] == 0) Enemies[i]->substructure[1] = 1;
                if(samvar.dim.bot < Enemies[i]->bound.bot) Enemies[i]->substructure[1] = 1;
            }
            else if(Enemies[i]->coord.orientation == DOWN)
            {
                Move_Enemy(i, 0, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft, Enemies[i]->bound.rgt, Enemies[i]->bound.bot+1, 0x02))
                {
                    Enemies[i]->substructure[1] = 1;
                }
                if(samvar.dim.lft > Enemies[i]->bound.rgt AND Enemies[i]->substructure[5] == 0) Enemies[i]->substructure[1] = 1;
                if(samvar.dim.bot < Enemies[i]->bound.bot) Enemies[i]->substructure[1] = 1;
            }
            else if(Enemies[i]->coord.orientation == RIGHT)
            {
                Move_Enemy(i, 1, Enemies[i]->substructure[3]-4);
                if(Scan_Clips_X(Enemies[i]->bound.lft+3, Enemies[i]->bound.rgt-3, Enemies[i]->bound.bot+1, 0x02))
                {
                    Enemies[i]->substructure[5] = 1;
                }
                if(Scan_Clips_Y(Enemies[i]->bound.top, Enemies[i]->bound.bot, Enemies[i]->bound.rgt+1, 0x02))
                {
                    Enemies[i]->coord.orientation = LEFT;
                    Enemies[i]->substructure[5] = 1;
                }
                if(samvar.dim.rgt < Enemies[i]->bound.lft AND Enemies[i]->substructure[5] == 0) Enemies[i]->substructure[1] = 1;
                if(samvar.dim.bot < Enemies[i]->bound.bot) Enemies[i]->substructure[1] = 1;
            }
        }
    }
}
