/*============================================================================*/
/* SaltyString.h, an include file of useful string functions                  */
/* Designed by SaltyJustice                                                   */
/* Email: SaltyJustice@hotmail.com  Homepage: metroidclassic.blogspot.com     */
/*============================================================================*/

#include <string.h>

#define AND &&
#define OR ||

//Declarations
void strclr(char *string, int size);
void strclr_start(char *string, int size, int start);
void strnscat(char *string1, char *string2, int length, int start, char char_stop);
void strnsecat(char *string1, char *string2, int start, int end);
void Init_String(char *string, char text[], int len);
int Char_Search(char string[], char letter);
int Inverse_Char_Search(char string[], char letter);
int Char_Search_Placed(char string[], char letter, int start, int end);
void String_Shift(char *string, int shifts);
void String_Shift_Placed(char *string, int shifts, int start);
void Refine_String(char *string, int words);
int Total_Words(char string[]);
int Get_Word(char *string1, char string2[], int number);
void Remove_Quotes(char *string);
int Find_Inst_Char_NDupe(char string[], char letter, int occurances);
char **Collect_All_Words(char string[], int *len);
char **Collect_All_Words_Brackets(char string[], int *len);
int Find_Word(char **array, char word[], int maxsize, int start);
int Find_Lowest(int args, ...);
int Find_Lowest_NonNeg(int args, ...);
int strncasecmp_start(int start, char string1[], char string2[], int length);

void strclr(char *string, int size)
{
    if(!size)
        size = strlen(string);
    memset(string, 0, size);
}

void strclr_start(char *string, int size, int start)
{
    int loop;
    if(!size)
    {
        size = strlen(string) - start;
    }
    for(loop = start; loop < size; loop ++)
    {
        string[loop] = 0;
    }
}

//Used to Append length characters from string2 to string1, starting at start
void strnscat(char *string1, char *string2, int length, int start, char char_stop)
{
    int c;
    int cstart = strlen(string1);
    for(c = start; c < start + length; c ++)
    {
        if(string2[c] == char_stop OR string2[c] == 0)
        {
            c = start + length;
        }
        else
        {
            string1[cstart + c - start] = string2[c];
        }
    }
}

//Same as strnscat above, but does not use char_stop
void strnsecat(char *string1, char *string2, int start, int end)
{
    int c;
    if(start > end)
    {
        int temp = start;
        start = end;
        end = temp;
    }
    if(start == end)
    {
        string1[0] = string2[start];
    }
    for(c = start; c < end; c ++)
    {
        string1[c - start] = string2[c];
    }
}

//Starts a string
void Init_String(char *string, char text[], int len)
{
    if(!len) len = strlen(text)+1;
	else if (len>strlen(string)) len=strlen(string);
    for(int l = 0; l < len; l ++) string[l] = 0;

    if(strlen(text)) strcat(string, text);
}

//Finds the [occurances]th position that letter is in
int Find_Inst_Char(char string[], char letter, int occurances)
{
    int len = strlen(string);
    int spot = 0;
    while(occurances > 0 AND spot < len)
    {
        if(string[spot] == letter)
        {
            occurances --;
        }
        spot ++;
    }
    if(spot == len)
    {
        return 0;
    }
    return spot;
}

//Finds the first spot that is the letter
int Char_Search(char string[], char letter)
{
    int x;
    for(x = 0; x < strlen(string); x ++)
    {
        if(string[x] == letter)
        {
            return x;
        }
    }
    return strlen(string);
}

int Char_Search_Placed(char string[], char letter, int start, int end)
{
    int x;
    if(start != end)
    {
        if(start > end)
        {
            int temp = start;
            start = end;
            end = temp;
        }
        for(x = start; x < end; x ++)
        {
            if(string[x] == letter)
            {
                return x;
            }
        }
    }
    return strlen(string);
}

//Find the first spot that isn't the letter
int Inverse_Char_Search(char string[], char letter)
{
    int x;
    for(x = 0; x < strlen(string); x ++)
    {
        if(string[x] != letter)
        {
            return x;
        }
    }
    return strlen(string);
}

int Inverse_Char_Search_Placed(char string[], char letter, int start, int end)
{
    int x;
    if(start != end)
    {
        if(start > end)
        {
            int temp = start;
            start = end;
            end = temp;
        }
        for(x = start; x < end; x ++)
        {
            if(string[x] != letter)
            {
                return x;
            }
        }
    }
    return strlen(string);
}

//Moves the string by shifts characters to the left
//e.g. after 2 shifts, "Hello" becomes "llo"
void String_Shift(char *string, int shifts)
{
    int x;
	const int l = strlen(string);
    for(x = 0; x < l + 1 - shifts; x ++)
    {
        string[x] = string[x + shifts];
    }
}

void String_Shift_Placed(char *string, int shifts, int start)
{
    int x;
	const int l = strlen(string);
    for(x = start; x < l + 1 - shifts; x ++)
    {
        string[x] = string[x + shifts];
    }
}

//Refine means remove all doubled spaces, trailing spaces, and leading spaces.
//If words == 0, then this function will scrap all leading spaces.
void Refine_String(char *string, int words)
{
    int leading = Inverse_Char_Search(string, ' ');
    String_Shift(string, leading);
    int i = 0;
    int position = 0;
    int start;
    while(i < words)
    {
        start = Char_Search_Placed(string, ' ', position, strlen(string));
        leading = Inverse_Char_Search_Placed(string, ' ', start, strlen(string)) - 1;
        String_Shift_Placed(string, leading - start, start);
        i ++;
    }
}

//This function assumes that a word is something seperated from the others by a space
//It will also return how many words it went through.  If you enter a super high number
//you can get back how many words there are total.
int Get_Word(char *string1, char string2[], int number)
{
    int len = strlen(string2);
    int words = 0;
    char string2copy[len];
    char string2dupe[80];
    Init_String(string2copy, string2, len);
    int start, end;
    int position = 0;
    for(int i = 0; i < number - 1; i ++)
    {
        start = Inverse_Char_Search_Placed(string2copy, ' ', 0, strlen(string2copy));
        end = Char_Search_Placed(string2copy, ' ', start, strlen(string2copy));
        String_Shift(string2copy, end);
        if(!strcmp(string2copy, string2dupe))
        {
            i = number - 1;
            words ++;
        }
        else
        {
            Init_String(string2dupe, string2copy, 80);
            words ++;
        }
    }
    start = Inverse_Char_Search_Placed(string2copy, ' ', 0, strlen(string2copy));
    end = Char_Search_Placed(string2copy, ' ', start, strlen(string2copy));
    string2copy[end] = 0;
    strclr(string1, 0);
    len = strlen(string2copy);
    if(end != -1)
    {
        strnsecat(string1, string2copy, start, end);
    }
    else
    {
        string1[0] = string2copy[0];
    }
    return words;
}

void Remove_Quotes(char *string)
{
    if(string[0] == '"') String_Shift(string, 1);
    int q = Char_Search_Placed(string, '"', 0, strlen(string));
	if (q<strlen(string))
		string[q] = 0;
}

int Find_Inst_Char_NDupe(char string[], char letter, int occurances)
{
    int len = strlen(string);
    int spot = 0;
    int lastdet = -2;
    while(occurances > 0 AND spot < len)
    {
        if(string[spot] == letter)
        {
            if(lastdet == spot - 1)
            {
                lastdet = spot;
            }
            else
            {
                lastdet = spot;
                occurances --;
            }
        }
        spot ++;
    }
    if(spot == len)
    {
        return 0;
    }
    return spot;
}

//Takes an array, resizes it, and fills each slot with one word from string.
//It does this until all words are in the array.
char **Collect_All_Words(char string[], int *len)
{
    int pos   = 0;
    int opos  = 0;
    int words = 0;

    Refine_String(string, 0);

    if(string[0] == 10) return NULL;
    if(string[0] == 13) return NULL;

    char **array = (char **)malloc(sizeof(char *) * 16);

    while(pos < strlen(string))
    {
        opos = pos;
        pos = Char_Search_Placed(string, ' ', pos, strlen(string));
        words ++;

        //Declare size for new word
		if (words%16==0)
			array = (char **)realloc(array, sizeof(char *) * 16*(words/16+1));

        array[words-1] = (char *)malloc(sizeof(char) * (pos - opos + 1));

        strnsecat(array[words-1], string, opos, pos);
        array[words-1][(pos-opos)] = 0;

        pos = Inverse_Char_Search_Placed(string, ' ', pos, strlen(string));
    }
    if(len != NULL) *len = words;
    return array;
}

//Same as above, but respects brackets as a single word
char **Collect_All_Words_Brackets(char string[], int *len)
{
    int pos   = 0;
    int opos  = 0;
    int words = 0;
    int brpos = 0;//Extra var for brackets

    Refine_String(string, 0);

    if(string[0] == 10)
    {
        *len = 0;
        return NULL;
    }

    char **array  = (char **)malloc(sizeof(char *));
    if(array == NULL) return NULL;

    while(pos < strlen(string))
    {
        opos = pos;
        pos = Char_Search_Placed(string, ' ', pos, strlen(string));
        brpos = Char_Search_Placed(string, '(', opos, strlen(string));
        if(brpos > opos AND brpos < pos)//True if the bracket is in the current word
        {
            brpos = Char_Search_Placed(string, ')', opos, strlen(string));
            pos = Char_Search_Placed(string, ' ', brpos, strlen(string));
        }
        words ++;

        //Declare size for new word
		if ((words+1)%16==0)
			array = (char **)realloc(array, sizeof(char *) * 16*((words+1)/16+1));
        if(array == NULL) return NULL;

        array[words-1] = (char *)malloc(sizeof(char) * ((pos - opos) + 1));
        array[words-1][(pos - opos)] = 0;
        if(array[words-1] == NULL) return NULL;

        strnsecat(array[words-1], string, opos, pos);
        array[words-1][pos - opos] = 0;
        array[words-1][(pos-opos)] = 0;

        pos = Inverse_Char_Search_Placed(string, ' ', pos, strlen(string));
    }
    if(len != NULL) *len = words;
    return array;
}

//Using an array where each cell is a string, find a word and return its location.
//Best used with above function.
int Find_Word(char **array, char word[], int maxsize, int start)
{
    for(int i = start; i < maxsize; i ++)
    {
        if(!strcasecmp(array[i], word))
        {
            return i;
        }
    }
    return maxsize+1;
}

//All passes values must be of type int or comparable.
int Find_Lowest(int args, ...)
{
    va_list argptr;
    va_start(argptr, args);
    int lowest = va_arg(argptr, int);

    for(int i = 0; i < args-1; i ++)
    {
        int n = va_arg(argptr, int);
        if(n < lowest) lowest = n;
    }

    return lowest;
}

//Same as above, returns 0 if
int Find_Lowest_NonNeg(int args, ...)
{
    va_list argptr;
    va_start(argptr, args);
    int lowest = -1;

    for(int i = 0; i < args; i ++)
    {
        int n = va_arg(argptr, int);
        if((n < lowest OR lowest == -1) AND n >= 0) lowest = n;
    }

    return lowest;
}

int strncasecmp_start(int start, char string1[], char string2[], int length)
{
    for(int i = start; i < length + start; i ++)
    {
        if(string1[i] > string2[i-start]) return 1;
        else if(string1[i] < string2[i-start]) return -1;
    }
    return 0;
}
