////////////////////////////////////////////////////////////////////////////////
// Shriekbats
//
// Shriekbat substructure
// 0 = If true, the bat is diving on Samus
//
////////////////////////////////////////////////////////////////////////////////

int Create_Shriekbat(int x, int y, bool hardtype)
{
    for(int i = 0; i < total_enemies; i ++) //Scan the old enemies to see if one is unused
    {
        if(!Enemies[i]->active)
        {
            Place_Shriekbat_In_Slot(i, x, y, hardtype);
            return i;
        }
    }
    Enemies = (ENEMY **)realloc(Enemies, sizeof(ENEMY *) * (total_enemies + 1));
    Enemies[total_enemies] = (ENEMY *)malloc(sizeof(ENEMY));
    Enemies[total_enemies]->substructure = NULL;
    Enemies[total_enemies]->display.images = NULL;
    Enemies[total_enemies]->immune.codes = NULL;

    Place_Shriekbat_In_Slot(total_enemies, x, y, hardtype);
    total_enemies ++;
    return total_enemies - 1;
}

void Place_Shriekbat_In_Slot(int i, int x, int y, bool hardtype)
{
    Enemies[i]->active = true;
    Enemies[i]->type_flag = SHRIEKBAT;
    Enemies[i]->funcptr = &Update_Shriekbat;
    if(Enemies[i]->substructure) free(Enemies[i]->substructure);
    Enemies[i]->size_of_substructure = 2;
    Enemies[i]->substructure = (unsigned char*)malloc(sizeof(unsigned char) * Enemies[i]->size_of_substructure);
    Enemies[i]->substructure[0] = 0;
    Enemies[i]->substructure[1] = 0;
    Enemies[i]->coord.x = x;
    Enemies[i]->coord.y = y;
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 16;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 24;
    Enemies[i]->coord.orientation = RIGHT;
    Enemies[i]->stats.health = 2;
    if(hardtype) Enemies[i]->stats.health *= 2;
    Enemies[i]->stats.hardtype = hardtype;
    Enemies[i]->stats.damage = 5;
    Enemies[i]->stats.speed_modifier = 1;
    Enemies[i]->stats.vulnerabilities = BEAM_ALL;
    if(Enemies[i]->display.images) free(Enemies[i]->display.images);
    Enemies[i]->display.images = (CUSTOM_IMAGE **)malloc(sizeof(CUSTOM_IMAGE *) * 2);
    Enemies[i]->display.images[0] = Enemy_Images[5];
    Enemies[i]->display.images[1] = Enemy_Images[6];
    Enemies[i]->display.to_show = 0;
    Enemies[i]->display.rotation = 0;
    Enemies[i]->display.draw_cnt = 0;
    Enemies[i]->team = 1;
    Enemies[i]->Sound = &Audio.Injured_Small;
    Enemies[i]->stun_timer = 0;
    Enemies[i]->freeze_timer = 0;
    Enemies[i]->refreeze_timer = 0;
    Add_To_Target_List(TARGET_ENEMY, i);

    if(Enemies[i]->immune.codes) free(Enemies[i]->immune.codes);
    Enemies[i]->immune.codes = NULL;
    Enemies[i]->immune.total_codes = 0;
}

void Update_Shriekbat(int i)
{
    //Hit detection and death sequence
    if(Enemies[i]->stats.health <= 0) //Shriekbat is dead :(
    {
        Create_Random_Item(Enemies[i]->bound, SMALLTYPE, false);
        int spritenum = Create_Sprite(((Enemies[i]->bound.lft+Enemies[i]->bound.rgt)/2)-16, ((Enemies[i]->bound.top+Enemies[i]->bound.bot)/2)-16, 3, 2, 6);
        if(spritenum != -1) Imagize_Sprite_Explosion(spritenum);
        Enemies[i]->active = false;
        Remove_From_Target_List(&Enemies[i]->stats.health);
        Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        return;
    }
    if(Enemies[i]->stun_timer > 0)
    {
        Enemies[i]->stun_timer --;
        return;
    }
    if(Enemies[i]->freeze_timer > 0)
    {
        Freeze_Clip_Area(Enemies[i]->bound, STOP_FREEZE);
        Enemies[i]->freeze_timer --;
        if(Enemies[i]->freeze_timer == 0) Unclip_Area(Enemies[i]->bound, STOP_FREEZE);
        if(Enemies[i]->freeze_timer < 50)
        {
            if((Enemies[i]->freeze_timer % 4) < 2)
            {
                Enemies[i]->palette = 2;
            }
            else
            {
                if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
                else Enemies[i]->palette = 0;
            }
        }
        else
        {
            Enemies[i]->palette = 2;
        }
        return;
    }
    if(Enemies[i]->stats.hardtype) Enemies[i]->palette = 1;
    else Enemies[i]->palette = 0;

    //Clipping information
    Enemies[i]->bound.lft = Enemies[i]->coord.x;
    Enemies[i]->bound.rgt = Enemies[i]->coord.x + 16;
    Enemies[i]->bound.top = Enemies[i]->coord.y;
    Enemies[i]->bound.bot = Enemies[i]->coord.y + 24;

    //Drawing Code
    Enemies[i]->display.draw_cnt ++;
    if(Enemies[i]->substructure[0] == 1 OR Enemies[i]->substructure[0] == 2) Enemies[i]->display.draw_cnt ++; //Increment by 2 when falling
    if(Enemies[i]->display.draw_cnt >= 24) Enemies[i]->display.draw_cnt = 0;

    if(Enemies[i]->display.draw_cnt < 6)
    {
        Enemies[i]->display.to_show = 0;
        Enemies[i]->display.hflip = false;
    }
    else if(Enemies[i]->display.draw_cnt < 12)
    {
        Enemies[i]->display.to_show = 1;
        Enemies[i]->display.hflip = false;
    }
    else if(Enemies[i]->display.draw_cnt < 18)
    {
        Enemies[i]->display.to_show = 0;
        Enemies[i]->display.hflip = true;
    }
    else if(Enemies[i]->display.draw_cnt < 24)
    {
        Enemies[i]->display.to_show = 1;
        Enemies[i]->display.hflip = true;
    }
    Enemies[i]->display.vflip = false;

    //Idle mode
    if(Enemies[i]->substructure[0] == 0)
    {
        if((samvar.dim.lft > Enemies[i]->bound.lft - 42 AND samvar.dim.lft < Enemies[i]->bound.lft) OR
           (samvar.dim.rgt < Enemies[i]->bound.rgt + 42 AND samvar.dim.rgt > Enemies[i]->bound.rgt))
        {
            Enemies[i]->substructure[0] = 1; //Goes into active mode
        }
    }

    //Active mode
    else if(Enemies[i]->substructure[0] == 1)
    {
        if(samvar.dim.rgt < Enemies[i]->bound.lft) Enemies[i]->coord.x --;
        else if(samvar.dim.lft > Enemies[i]->bound.rgt) Enemies[i]->coord.x ++;
        for(int q = 0; q < 3; q ++)
        {
            if(Scan_Clips_X(Enemies[i]->bound.lft, Enemies[i]->bound.rgt, Enemies[i]->bound.bot, STOP_ENEMY))
            {
                Enemies[i]->substructure[0] = 2;
                q = 3;
            }
            else
                Enemies[i]->coord.y += 1;
        }
    }

    //Detonation mode
    else if(Enemies[i]->substructure[0] == 2)
    {
        Enemies[i]->substructure[1] ++;
        if(Enemies[i]->substructure[1] >= 40)
        {
            Enemies[i]->active = false;
            Remove_From_Target_List(&Enemies[i]->stats.health);
            int q = Create_Sprite(Enemies[i]->bound.lft+8, Enemies[i]->bound.top+12, 1, 1, 15);
            Imagize_Sprite_Shriekbat_Orb(q, -2, 0);

            q = Create_Sprite(Enemies[i]->bound.lft+8, Enemies[i]->bound.top+12, 1, 1, 15);
            Imagize_Sprite_Shriekbat_Orb(q, -1, -1.732);

            q = Create_Sprite(Enemies[i]->bound.lft+8, Enemies[i]->bound.top+12, 1, 1, 15);
            Imagize_Sprite_Shriekbat_Orb(q, 1, -1.732);

            q = Create_Sprite(Enemies[i]->bound.lft+8, Enemies[i]->bound.top+12, 1, 1, 15);
            Imagize_Sprite_Shriekbat_Orb(q, 2, 0);
        }
    }
}
