///////////////////////////////////////////////////////////////////////////////
// Custom IPF and ISF Stuff
///////////////////////////////////////////////////////////////////////////////
typedef struct
{
    int xsize, ysize;
    char **image;
    int colors, palettes;
    int **values;
}CUSTOM_IMAGE;

typedef struct
{
    char *name;
    int position;
}INDEX;

typedef struct
{
    char version_info[20]; //20 bytes
    int total_images;
    INDEX *image_index;
}MCI_FILE;
///////////////////////////////////////////////////////////////////////////////
// Script Stuff
///////////////////////////////////////////////////////////////////////////////
typedef struct
{
    char *str;
    int len;
}LINE;

typedef struct
{
    LINE *line;
    int total_lines;
    char internal_name[80];
}SCRIPT;

///////////////////////////////////////////////////////////////////////////////

typedef struct
{
    char name[8];
    char *key;
    char askey;
    char shkey;
    bool repress;
}KEYLIST;

typedef struct
{
    char name[10];
    KEYLIST *key;
    bool pressing;
}CONTROLLIST;

typedef struct //Contains bounding boxes and their sizes from 0
{
    int lft, rgt, top, bot;
}DIMENSION;


typedef struct
{
    int posinfo[7]; //See Filetype1.1 for what each position is
    int xsize, ysize;//Standardly multiples of 16.
    int xpos, ypos;  //Where the room is on the minimap
    int **tiles;     //Size is [X][Y]
    int **clips;     //Size is [X][Y]
}ROOM;

typedef struct
{
    char *version;      //Should be 20 characters
    int total_rooms;
    ROOM *Rooms;
    int *room_pos_info; //Locations of the rooms in the file
    char **room_names;  //The names of the rooms extracted from the files
}CAMPAIGN;

typedef struct
{
    bool active;
    DIMENSION dim;
    int flags;
}CAMBOUND;

typedef struct
{
    bool active;
    char *name;
    DIMENSION dim;
    int facing;
    int openstatus;
    int to_reclose;
    int type;
    int health;
    int flags;
}DOOR;

typedef struct
{
    bool active;
    DIMENSION dim;
    char *roomto;
    char direction; //UP or DOWN
    int cur_y; //When 0, it is at it's DIMENSION.top position
    bool in_use; //When false, not in use.  Samus presses down/up it becomes 1
    int lockout;
}ELEVATOR;

typedef struct
{
    char *name;
    DOOR *doorhook; //If NULL then this zone does not hook to a door
    int doorhookval; //If -1 then this zone does not hook to a door
    DIMENSION dim;
    char *roomto;
    char *transto;
    int direction;
}TRANSITION;

typedef struct
{
    char *name;
    bool is_int_type;
    char *valuec;
    int valuei;
}CVAR;

typedef struct
{
    int owner_flag; //If -1, shot is inactive
    int duration; //If -1, shot has no duration
    int palette;
    DIMENSION dim;
    DIMENSION clip;
    DIMENSION draw;
    DIMENSION clip_dif;
    int image_num;
    CUSTOM_IMAGE **images;
    int image_show;
    int image_numer;
    int image_denom;
    int img_offset_x;
    int img_offset_y;
    float xspeed, yspeed;
    float xremainder, yremainder;
    int damage;
    unsigned char properties; //See HP and Hit Detection notes
    int facing;
    bool terminate; //When true, projectile dies at the end of the frame
    bool missile_terminate; //If true and a missile, create the fireworks

    bool penetrate_enemy; //Beam punches through enemies
    bool penetrate_world; //Beam is not clipped
    bool bomb;
    bool reverse; //Beam travels down then up if true
    int timer;
    int lock_position;
    int vibrate_range;
    int width, height;
    bool helix_draw;
    int beamcode;
    int not_less_than_pos; //Drawing will not occur past this point in the opposite of facing
    int damage_cooldown; //Cannot deal damage until this ticks to 0.  Standardly is 3.
}PROJECTILE;

typedef struct
{
    bool active;
    int type; //See notes for what type can mean here
    int location; //The hard position, i.e 0, 1, 2, 3
    DIMENSION *dim;
    int *hp;
    int *team;
    int *stun_timer;
    int *vulnerabilities;
    HSTREAM *snd;
}TARGET;

typedef struct
{
    bool active;
    bool fanfare; //Special items, like morph ball/missile tanks
    char *string;
    int x, y;
    DIMENSION dim;
    int type; //See notes for the types
    int timer;
    int expiry;
    int breakable_hook; //Which position the breakable is in its array
    BITMAP *bmp0;
    BITMAP *bmp1;
    int position;
    bool super_type; //Metroid drops items worth 10x their normal value.
}ITEM;

typedef struct
{
    float xspeed, yspeed;
    float xremainder, yremainder;
    float xaccel, yaccel;
    float xmax, ymax, xmin, ymin;
    float xpos, ypos;
    float ymax_reserve;
}PHYSICS_PACK;


typedef struct
{
    bool lft, top, rgt, bot; //Do the corresponding effect when triggered in that direction
    bool terminate; //Destroy
    bool bounce; //Bounce at half power off
    bool friction; //If true, the object slows by .25 while in contact with the bottom
}CLIPPING_PACK;

typedef struct
{
    bool hflip, vflip;
    fixed angle;
}DRAWING_PACK;

typedef struct
{
    int damage;
}DAMAGE_PACK;

typedef struct
{
    bool reverse_roll;//If xspeed > 0, object rolls clockwise.  If this is true, reverse that.
    bool is_rolling;//When false, check for impact.  When true, do the rolling code.
    int roll_timer;//Becomes 0 at impact time.  Is -1 before impact.
}ROLLING_PACK;

typedef struct
{
    bool active;
    int palette;
    DIMENSION dim;
    int frame_num;
    int frame_denom;
    CUSTOM_IMAGE **images;
    int total_images;
    int duration;

    PHYSICS_PACK *phys;
    CLIPPING_PACK *clip;
    DRAWING_PACK *draw;
    DAMAGE_PACK *damage;
    ROLLING_PACK *roll;
}SPRITE;

typedef struct
{
    bool active;
    int x, y;
    int images;
    int cooldown_num;
    int cooldown_denom;
    void (*hookfunc)(int);
}SPRITE_SPAWNER;

typedef struct
{
    int palettes; //How many recolors in this pack, lowest is 2.  Also, X dimension.
    int uniques;  //How many unique colors in the image, lowest is 1.  Y dimension.
    int **cols;
}RECOLOR_PACK;

typedef struct
{
    int x, y;
    char **values;
}SUB_BITMAP;

typedef struct
{
    bool active;
    int x, y;
    DIMENSION dim;
    int respawn_timer; //If -1, block is solid
    int respawn_max;
    int flags;
    bool killflag; //If true, block does not respawn
    int fadein_timer;
    int fadeout_timer;
    int tile_show;
    bool show_item;
}BREAKABLE;

typedef struct
{
    bool active;
    int x, y;
    int color;
}PIXEL;

typedef struct
{
    bool active;
    int x, y;
    int timer;
}SAMSTART;

//// File Saving /////////
typedef struct //This struct keeps track of missile doors
{
    int roomname_len; //Add one for trailing null
    char *roomname;
    int doorname_len;
    char *doorname;
    int hit_status; //Number of hits remaining (0 means it should spawn blue)
}OPENED_DOOR;

typedef struct
{
    int roomname_len; //Add one for trailing null
    char *roomname;
    int object_pos;
}SAVED_OBJECT;

typedef struct
{
    int name_len;//Add one for trailing null
    char *name;
    int xsize, ysize;
    short **vals;
}SAVEMAP;

typedef struct
{
    int roomname_len; //Add one for trailing null
    char *roomname;

    int energy, energy_max;
    int missiles, missiles_max;
    int long_beam, charge_beam, ice_beam, wave_beam, spazer_beam, plasma_beam;
    int helix_beam, burn_beam, shock_beam;
    int varia_suit, gravity_suit;
    int morph_ball, bombs, spring_ball, screw_attack;
    int high_jump, space_jump, speed_booster;

    bool ridley_dead, kraid_dead;

    int doors_saved;
    OPENED_DOOR *Door_Status_Save;
    int objects_saved;
    SAVED_OBJECT *Object_Status_Save;

    int maps_accessed;
    SAVEMAP *Savemap;
}SAVEFILE;

typedef struct
{
    bool active;
    DIMENSION dim;
    int cooldown; //If 0, you can save
    bool show_message; //"Press F3 to save"
    bool show_message_saved; //"Game saved!"
}SAVEZONE;

typedef struct
{
    bool active;
    bool has_top;
    DIMENSION dim;
    int type; //0 = Water, 1 = Acid, 2 = Lava
    int counter;
    BITMAP *bmp;
    BITMAP *bmp2;
}WATER_ZONE;

typedef struct
{
    DIMENSION dim;
    char *text;
    bool *col;
    bool *use;
    int lpath, upath, rpath, dpath;
    int *exclusive;
    int exclusive_total;
}SUBSCREEN_ITEM;

typedef struct
{
    char *name;
    int xsize, ysize;
    short **vals;
    BITMAP *base;
}MAP;

typedef struct
{
    int start_frame;
    int end_frame;
    int rot_numer;
    int rot_denom;
}CLUSTER;

typedef struct
{
    CUSTOM_IMAGE **Img;
    int *offx;
    int *offy;
}SAMUS_FRAMES;
